/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiDrawContext_h__
#define __nuiDrawContext_h__

#include "ngl.h"
#include "nglFont1.h"
#include "nuiRect.h"
#include "nuiShape.h"
#include "nuiWidget.h"
#include "nuiTexture.h"
#include "nuiTheme.h"

#include <stack>


#define NUI_MAX_TEXTURE_SETS 8
#define NUI_MAX_LIGHTS       8
#define NUI_MAX_CLIP_PLANES  8
        
/** This class is a struct like container that hold a cache of an opengl rendering context (main paremeters anyway).
*/
class nuiRenderState
{
public:
  nuiRenderState();
  nuiRenderState(const nuiRenderState& rState);
  virtual ~nuiRenderState();
  void GetGLState(); ///< Mirror the real current opengl state.


  // glEnable / glDisable cache
  bool mLighting;
  bool mFog;
  bool mBlending;
  bool mAlphaTest;
  bool mAutoNormal;
  bool mColorMaterial;
  bool mCullFace;
  bool mDepthTest;
  bool mDither;
  bool mPointSmooth;
  bool mLineSmooth;
  bool mLineStipple;
  bool mScissorTest;
  bool mStencilTest;

  bool mTexture2D;
  bool mTexture1D;
  bool mTextureGenQ;
  bool mTextureGenR;
  bool mTextureGenS;
  bool mTextureGenT;

  bool mLogicOp; 

  bool mMap1Color4; 
  bool mMap1Index; 
  bool mMap1Normal; 
  bool mMap1TextureCoord1;
  bool mMap1TextureCoord2; 
  bool mMap1TextureCoord3; 
  bool mMap1TextureCoord4; 
  bool mMap1Vertex3;
  bool mMap1Vertex4; 

  bool mMap2Color4;
  bool mMap2Index; 
  bool mMap2Normal; 
  bool mMap2TextureCoord1;
  bool mMap2TextureCoord2; 
  bool mMap2TextureCoord3; 
  bool mMap2TextureCoord4; 
  bool mMap2Vertex3;
  bool mMap2Vertex4; 

  bool mNormalize;
  bool mPolygonSmooth;
  bool mPolygonStipple; 

  bool mpLight[NUI_MAX_LIGHTS];
  bool mpClipPlane[NUI_MAX_CLIP_PLANES];

  GLenum mBlendSourceFactor;
  GLenum mBlendDestinationFactor;

  // glEnableClientState / glDisableClientState cache:
  bool mVertexPointer;
  bool mNormalPointer;
  bool mColorPointer;
  bool mTexCoordPointer[ NUI_MAX_TEXTURE_SETS ];

  // For small optims:
  bool mGroup1Changed;
  bool mGroup2Changed;
  bool mGroup3Changed;

  // Actual drawing params:
  nuiShapeMode mShapeMode;
  nuiColor mStrokeColor;
  nuiColor mFillColor;
  nuiSize mLineWidth;
  nuiTexture* mpCurrentTextures[ NUI_MAX_TEXTURE_SETS ];
  nuiShape::Winding mWinding;

  nuiColor mGradientColor1;
  nuiColor mGradientColor2;
  nglVector2f mGradientPoint1;
  nglVector2f mGradientPoint2;

  bool mAntialiasedLines;
};


/// This class permits to abstract the opengl rendering context in order to minimize context switching overhead. It is far from complete at the moment. I hope to be able to move most opengl low level rendering code to it one day.
class nuiDrawContext
{
public:
  nuiDrawContext(const nglPath& rResPath, const nuiRect& rRect);
  virtual ~nuiDrawContext();

  virtual void PushClipping();
  virtual void PopClipping();
  virtual bool Clip(nuiWidgetPtr pWidget, const nuiRect& rLocalRect, bool IgnorePosition = true); ///< This method restricts all the rendering operation to the given local rectangle.
  virtual bool Clip(nuiWidgetPtr pWidget); ///< This method restricts all the rendering operation to the client rectangle.
  virtual bool SetClipping(bool set); ///< Enables or disable clipping.
  virtual bool ResetClipRect(); ///< This method reset the clipping rect to the original size of the window.

  virtual void SetSize(int w,int h); ///< Set the size of the drawing surface (for clipping calculus).

  // Theme:
  nuiTheme* GetTheme();
  void SetTheme(nuiTheme* pTheme, bool OwnTheme = true);

  /* @name Render state manipulation */
  //@{
  void PushRenderState();
  void PopRenderState();

  bool EnableLighting (bool val);
  bool EnableFog (bool val);
  bool EnableBlending (bool val);
  bool EnableAlphaTest (bool val);
  bool EnableAutoNormal (bool val);
  bool EnableColorMaterial (bool val);
  bool EnableCullFace (bool val);
  bool EnableDepthTest (bool val);
  bool EnableDither (bool val);

  bool EnablePointSmooth (bool val);
  bool EnableLineSmooth (bool val);
  bool EnableLineStipple (bool val);
  bool EnableNormalize (bool val);
  bool EnablePolygonSmooth (bool val);
  bool EnablePolygonStipple (bool val);
 
  bool EnableScissorTest (bool val);
  bool EnableStencilTest (bool val);

  bool EnableTexture2D (bool val);
  bool EnableTexture1D (bool val);
  bool EnableTextureGenQ (bool val);
  bool EnableTextureGenR (bool val);
  bool EnableTextureGenS (bool val);
  bool EnableTextureGenT (bool val);

  bool EnableLogicOp (bool val);

  bool EnableMap1Color4 (bool val);
  bool EnableMap1Index (bool val);
  bool EnableMap1Normal (bool val);
  bool EnableMap1TextureCoord1 (bool val);
  bool EnableMap1TextureCoord2 (bool val);
  bool EnableMap1TextureCoord3 (bool val);
  bool EnableMap1TextureCoord4 (bool val);
  bool EnableMap1Vertex3 (bool val);
  bool EnableMap1Vertex4 (bool val);

  bool EnableMap2Color4 (bool val);
  bool EnableMap2Index (bool val);
  bool EnableMap2Normal (bool val);
  bool EnableMap2TextureCoord1 (bool val);
  bool EnableMap2TextureCoord2 (bool val);
  bool EnableMap2TextureCoord3 (bool val);
  bool EnableMap2TextureCoord4 (bool val);
  bool EnableMap2Vertex3 (bool val);
  bool EnableMap2Vertex4 (bool val);

  bool EnableLight (int light,bool val);
  bool EnableClipPlane (int num,bool val);
  
  bool EnableVertexPointer (bool val);
  bool EnableNormalPointer (bool val);
  bool EnableColorPointer (bool val);
  bool EnableTexCoordPointer (int num,bool val);

  bool SetBlendFunc(GLenum sfactor, GLenum dfactor);

  bool IsTextureCurrent  ( nuiTexture* pTex , int num = 0);
  nuiTexture* GetTextureCurrent ( int num = 0);
  bool SetTextureCurrent ( nuiTexture* pTex , int num = 0);

  bool ActivateRenderState(const nuiRenderState& rState);
  bool ResetRenderState(); ///< Set the render state to OpenGL's default values.

  const nuiRenderState& GetRenderState();
  void GetGLRenderState(); ///< Mirror the real current opengl state in the current nuiRenderState object.
  //@}

  /* @name Rendering helper methods */
/*
  //@{
  void DrawRect(nuiRect::DrawMode Mode, const nuiRect& rRect, nuiColor* pFillColor = NULL, nuiColor* pBorderColor = NULL); ///< Draw a rectangle.
  void DrawLine(float x1, float y1, float x2, float y2, nuiColor* pColor = NULL); ///< Draw a line.
  void DrawLine(int x1, int y1, int x2, int y2, nuiColor* pColor = NULL); ///< Draw a line.
  void DrawLines(uint count, nglVector2f* pPoints, nuiColor* pColor = NULL); ///< Draw a set of distinct lines.
  void DrawLineLoop(uint count, nglVector2f* pPoints, nuiColor* pColor = NULL); ///< Draw a set of connected lines.
  void DrawLineLoop(uint count, nglVector2f* pPoints, const nuiColor& rColor); ///< Draw a set of connected lines.
  void DrawLineStrip(uint count, nglVector2f* pPoints, nuiColor* pColor = NULL); ///< Draw a set of connected lines.
  void DrawLineStrip(uint count, nglVector2f* pPoints, const nuiColor& rColor); ///< Draw a set of connected lines.
  //@}
*/

  virtual void SetFillColor(const nuiColor& rColor);
  virtual void SetStrokeColor(const nuiColor& rColor);
  virtual const nuiColor& GetFillColor();
  virtual const nuiColor& GetStrokeColor();
  virtual void SetFillColor(nuiWidgetElement Element);
  virtual void SetStrokeColor(nuiWidgetElement Element);

  virtual void SetShapeGradient(const nuiColor& rColor1, const nuiColor& rColor2, const nglVector2f& rPoint1, const nglVector2f& rPoint2);
  virtual void GetShapeGradient(nuiColor& rColor1, nuiColor& rColor2, nglVector2f& rPoint1, nglVector2f& rPoint2);

  virtual void SetLineWidth(nuiSize Width);
  virtual void SetAntialiasedLines(bool set);
  virtual bool GetAntialiasedLines();

  virtual void SetWinding(nuiShape::Winding Rule); ///< Set the shapes filling/rasterization rule. This one is used only if the rendering shape have their rule set to nuiShape::eNone
  virtual nuiShape::Winding GetWinding(); ///< Get the shapes filling/rasterization rule.

  enum TextureMode
  {
    eNoTexture,
    eShapeCoords,
    eGenPlane
  };

  enum ColorMode
  {
    eContextColors,
    eShapeColors,
    eGradientColors
  };

  virtual void DrawShape(nuiShape* pShape, nuiShapeMode Mode, ColorMode UseColors = eContextColors, TextureMode UseTexture = eNoTexture); ///< Call StrokeShape and FillShape.
  
  // Fast and simple drawings:
  virtual void DrawRect(const nuiRect& Rect, nuiShapeMode Mode);
  virtual void DrawLine(const nglVector2f& rOrigin, const nglVector2f& rDestination);
  virtual void DrawLine(float x1, float y1, float x2, float y2);
  virtual void Clear(); ///< Initializes the current drawing buffer with the default values.
  virtual void SetClearColor(const nuiColor& ClearColor); ///< Set the color used to clear the background with Clear().
  virtual void DrawImage(const nuiRect& rDest, const nuiRect& rSource);
  virtual void DrawShade (const nuiRect& rSourceRect, const nuiRect& rShadeRect); ///< Draw a shade extending from SourceRect to ShadeRect. The shade may look quite diferent depending on the actual renderer.
  virtual void BlurRect(const nuiRect& rRect, uint Strength); ///< Blur a rectangle on screen.

  /** @name Matrix operations: */
  //@{
  virtual void PushMatrix();
  virtual void PopMatrix();
  virtual void LoadMatrix(const nglMatrixf& Matrix);
  virtual void MultMatrix(const nglMatrixf& Matrix);
  virtual void Translate(const nglVector2f& Vector);
  virtual void Translate(nuiSize X, nuiSize Y);
  virtual void Scale(nuiSize X, nuiSize Y);
  virtual void LoadIdentity();
  //@}

  /** @name Fonts and text handling: */
  //@{
  virtual void SetFont(nuiTheme::FontStyle Style, nglFont1* pFont, bool Delete=true); ///< Changes the font used for the given style. If Delete==true the old one will be deleted.
  virtual nglFont1* GetFont(nuiTheme::FontStyle Style); ///< Retreive the font used for the given style.
  virtual bool GetFontInfo(nuiTheme::FontStyle FontStyle, nglFontInfo1& rFontInfo);
  virtual int GetTextSize (nuiTheme::FontStyle FontStyle, float& rX, float& rY, const nglChar* pText); ///< Calculate the bounding of the string in texels and returns it in X & Y.
  virtual int GetTextPos (nuiTheme::FontStyle FontStyle, float X, const nglChar* pText); ///< Calculate the bounding of the char pos in the given text where the pixel (X,?) lies and returns it.

  virtual void SetTextColor(const nuiColor& rColor, nuiTheme::FontStyle Style = nuiTheme::Default);
  virtual void DrawText(nuiSize x, nuiSize y, const nglString& rString, nuiTheme::FontStyle Style = nuiTheme::Default); /// Draw text at the given coordinates and the given font style.
  //@}

protected:
  // Clipping:
  class nuiClipper : public nuiRect
  {
  public:
    nuiClipper(const nuiRect& rRect, bool enable = true)
      : nuiRect(rRect)
    {
      mEnabled = enable;
    }

    nuiClipper(bool enable = false)
    {
      mEnabled = enable;
    }

    nuiClipper(const nuiClipper& rClipper)
      : nuiRect(rClipper)
    {
      mEnabled = rClipper.mEnabled;
    }

    bool mEnabled;
  };

  virtual void ActivateClipper();

  std::stack <nuiClipper*> mpClippingStack;
  nuiClipper mClippingRect;

  // Render state stack:
  std::stack<nuiRenderState*> mpRenderStateStack;
  nuiRenderState mCurrentState;
  
  nuiSize mWidth,mHeight;

  nuiTheme* mpTheme;
  bool mOwnedTheme;

  nglPath mResPath;

  uint mMaxLights;
  uint mMaxClipPlanes;

  nuiColor mClearColor;

#ifndef CALLBACK 
  #define CALLBACK 
#endif

  static void CALLBACK StaticInternalTessBegin(GLenum type, void * polygon_data);
  static void CALLBACK StaticInternalTessEdgeFlag(GLboolean flag, void * polygon_data);
  static void CALLBACK StaticInternalTessVertex(void* vertex_data, void * polygon_data);
  static void CALLBACK StaticInternalTessEnd(void * polygon_data);
  static void CALLBACK StaticInternalTessCombine(GLdouble coords[3], void *vertex_data[4], GLfloat weight[4], void **outData, void * polygon_data);
  static void CALLBACK StaticInternalTessError(GLenum ErrNo, void * polygon_data);

  virtual void InternalTessBegin(GLenum type);
  virtual void InternalTessEdgeFlag(GLboolean flag);
  virtual void InternalTessVertex(void* vertex_data);
  virtual void InternalTessEnd();
  virtual void InternalTessCombine(GLdouble coords[3], void *vertex_data[4], GLfloat weight[4], void **outData);
  virtual void InternalTessError(GLenum ErrNo);

  // Temp Variables needed to build the tesselation cache:
  GLenum mCurrentTessMode;
  nuiShapeMode mCurrentTessShapeMode;
  nuiShape* mpCurrentTessShape;

  void GetGradientColor(const nglVector2f& rPoint, nuiColor& rColor);
  virtual void TesselateShape(nuiShape* pShape, nuiShapeMode Mode, ColorMode UseColors = eContextColors, TextureMode UseTexture = eNoTexture);
  class TempVert
  {
  public:
    nuiDrawContext* mpContext;
    nuiShape* mpShape;
    nglVector2f mVertex;
    nglVector2f mTexCoords;
    nuiColor mColor;
  };
  std::list<TempVert*> mpTessTempVertices; // These are the vertices dynamically created while tesselating a poly
  std::list<TempVert*> mpTessTempModeVertices; // These are the vertices to draw in the correct order.

  TextureMode mTessUseTexture;
  ColorMode mTessUseColors;
  virtual void DrawCachedShape(std::list<nuiShape::CacheElement*>* pElements, nuiShapeMode Mode, ColorMode UseColors, TextureMode UseTexture);

  nuiTexture* mpScratchPad;
  virtual nuiTexture* GetScratchPad(uint width, uint height);
};

#undef RENDERSTATE_ENABLE
#undef RENDERSTATE_ENABLECLIENT

#endif // __nuiDrawContext_h__
