/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiEditText_h__
#define __nuiEditText_h__

#include "nuiEvent.h"
#include "nuiComposite.h"
#include "nuiTheme.h"
#include "nuiTimer.h"

class nuiScrollBar;

/// This class implements the basic text edition widget in nui. It is the root of nuiEditLine and nuiText (which is not editable). For the moment it is very basic: word wrap is not implemented but selection & copy & paste is.
class nuiEditText : public nuiComposite
{
public:
  nuiEditText(nuiContainerPtr pParent, const nglString& rText);
  nuiEditText(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  virtual ~nuiEditText();

  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive);
  void InitProperties();

  virtual bool Draw(nuiDrawContext* pContext);

  virtual nuiRect CalcIdealSize();
  virtual bool SetRect(const nuiRect& rRect);

  // Text management:
  virtual bool SetText(const nglString& rText); ///< Set the text of the widget.
  virtual nglString GetText(); ///< Retreive the complete text of the widget.
  virtual nglString GetLine(int number); ///< Retreive a line of text.
  virtual bool AddText(const nglString& rText, int line=-1); ///< Happend text before the given line (default is to the end of the document).
  virtual bool InsertText(const nglString& rText, int line, int column); ///< Happend text at the given position.
  virtual bool DelLine(uint number); ///< Remove the given line from the widget.
  virtual bool DelSelection(); ///< Delete the currently selected text.
  virtual nglString GetSelection(); /// Return a string containing the current selection.
  virtual bool GetPixelPos(nuiSize X, nuiSize Y, int& rLine, int& rChar); ///< This method computes the position in the text corresponding to the point (X,Y) and returns it in rLine and rChar. Returns true on success.
  virtual bool IsInSelection(nuiSize X, nuiSize Y); ///< This returns true if the given pixel position is inside the selection mark. 

  virtual uint GetLineCount(); ///< Return the number of lines in the widget.

  virtual void SetFollowModifications(bool follow); ///< The text will always try to show the last modified line
  virtual bool GetFollowModifications(); ///< See SetFollowModifications

  // Appearance:
  virtual bool ShowLine(int number); ///< Make the given line of text visible.
  virtual bool ShowColumn(int number); ///< Make the given horizontal pixel of text visible.
  virtual bool SetFontStyle(nuiTheme::FontStyle FontStyle);
  virtual nuiTheme::FontStyle GetFontStyle();
  virtual void SetHScrollBar(bool flag);
  virtual void SetVScrollBar(bool flag);
  virtual bool GetHScrollBar();
  virtual bool GetVScrollBar();

  // Received events:
  virtual bool HScroll(nuiEvent* pEvent); ///< Connected to the SetValue event of the Horizontal scrollbar
  virtual bool VScroll(nuiEvent* pEvent); ///< Connected to the SetValue event of the Vertical scrollbar

  // Mouse:
  virtual bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseUnclicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseMoved  (nuiSize X, nuiSize Y);
  
  // Keyboard Events:
  virtual bool KeyDown (nglKeyCode Key, nglChar Char);
  virtual bool KeyUp       (nglKeyCode Key, nglChar Char);

  virtual bool BlinkTimer (nuiEvent* pEvent);
  virtual void OnSetFocus(nuiWidgetPtr pWidget);

  virtual void SetEditable(bool Editable);
  virtual bool IsEditable();

  // Text Drag and drop implementation:
  virtual bool OnCanDrop(nuiDragAndDrop* pDragObject,nuiSize X,nuiSize Y);
  virtual void OnCancelDrag(nuiDragAndDrop* pDragObject);
  virtual void OnDragged(nuiDragAndDrop* pDragObject);
  virtual void OnDropped(nuiDragAndDrop* pDragObject,nuiSize X,nuiSize Y, nglMouseFlags Button);

  virtual void Invalidate(bool LayoutChanged = false, bool TellChildren = false);

protected:
  class String : public nglString
  {
  public:
    String(nglString& rString)
      : nglString(rString)
    {
      mWidth = -1;
      mHeight = -1;
    }
    String(const String& rString)
      : nglString(rString)
    {
      mWidth = -1;
      mHeight = -1;
    }
    void Invalidate()
    {
      mWidth = -1;
      mHeight = -1;
    }
    nuiSize mWidth,mHeight; ///< This is the bounding box of the rendered text. We cache it for performence reasons.
  };

  nuiBool mFollowModifications;
  std::list<String> mpText;
  
  nuiInt32 mFirstVisibleLine;
  nuiInt32 mLastVisibleLine;

  nuiScrollBar* mpVScrollBar;
  nuiScrollBar* mpHScrollBar;

  nuiTheme::FontStyle mFontStyle;

  nuiSize mTextHeight;
  nuiSize mTextWidth;
  nuiSize mSelectGap;

  nuiUInt32 mCursorLine;
  nuiUInt32 mCursorColumn;
  bool mDisplayCursor; ///< Set by the cursor blink timer :).

  bool mKeyboardSelect; ///< Are we currently selecting with the keyboard? (shift key pressed).
  nuiUInt32 mSelectedColumn; ///< column of the Start of selection.
  nuiUInt32 mSelectedLine; ///< line of the Start of selection.
  nuiBool mSelection; ///< Is there an active selection?
  nuiBool mMakeCursorVisible; ///< Should we make the cursor visible on the next redraw?
  bool mPasting; ///< True if we are currently pasting text to the control.
  bool mClicked; ///< True if the widget was left clicked and not yet unclicked.

  nuiTimer* mpBlinkTimer;

  nuiBool mEditable;
  nuiEventSink<nuiEditText> mEditTextSink;
  nuiBool mHScrollBar;
  nuiBool mVScrollBar;

  nglTime mClickTime;
  bool mStartDrag;

  bool mRebuildGLList;
  GLint mGLList;
};

#endif // __nuiEditText_h__
