/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiEvent_h__
#define __nuiEvent_h__

#include "ngl.h"
#include <list>
#include "nuiEventRegistry.h"

class nuiEventSource;

/// This is the basic event object used in the user level event management system. 
class nuiEvent
{
public:
  nuiEvent(nuiEventSource* obj = NULL,int Type = -1);  
  virtual ~nuiEvent();

  int GetType();
  nuiEventSource* GetSource();

  void* mpUser; // This is the user defined data. It is defined on a connection basis.
private:
  int mType;
  nuiEventSource* mpSource;
};

/// You are never supposed to use this class directly!!! NEVER EVER :). 
class nuiEventTargetBase
{
public:
  virtual bool OnEvent(nuiEvent* pEvent) = 0;
  virtual void Disconnect(nuiEventSource& rSource) = 0;
};

/// Do not touch this either!
class nuiEventSource
{
public:  
  nuiEventSource();
  virtual ~nuiEventSource();

  virtual void Connect(nuiEventTargetBase* t);
  virtual void Disconnect(nuiEventTargetBase* t);
  virtual bool SendEvent(nuiEvent* event);
  virtual bool operator() (nuiEvent* pEvent=NULL);
  uint GetTargetCount();
  void EmptyTrash();

protected:
  std::list<nuiEventTargetBase*> mpTargets;
  std::list<nuiEventTargetBase*> mpTrash;
  bool mEnumerating;

private:
  // Restrict access to some methods & constructors:
  nuiEventSource(const nuiEventSource& rSource) ;
};


template <class T> class nuiEventTarget : public nuiEventTargetBase
{
public:
  typedef bool (T::*TargetFunc) (nuiEvent*);

  nuiEventTarget ()
  {
    mEnumerating = false;
    mpTarget = dynamic_cast<T*>(this);
  }

  virtual ~nuiEventTarget()
  {
    typename std::list<Link*>::iterator it;
    typename std::list<Link*>::iterator end = mpLinks.end();
    mEnumerating = true;
    for (it=mpLinks.begin(); it != end; it++)
    {
      Link* pLink = *it;
      pLink->mpSource->Disconnect(this);
      delete pLink;
    }
    mEnumerating = false;
    EmptyTrash();
  }

  virtual bool OnEvent(nuiEvent* pEvent)
  {
    bool handled = false;
    mEnumerating = true;
    typename std::list<Link*>::iterator it;
    typename std::list<Link*>::iterator end = mpLinks.end();
    for (it=mpLinks.begin(); it != end && !handled; it++)
    {
      if ( (*it)->mpSource == pEvent->GetSource())
      {
        typename std::list<TargetFunc>::iterator it2;
        typename std::list<TargetFunc>::iterator end2 = (*it)->mpTargets.end();

        for (it2 = (*it)->mpTargets.begin(); it2 != end2 && !handled; it2++)
        {
          TargetFunc pFunc = (*it2);
          pEvent->mpUser=(*it)->mpUser;
          // Beware!! If the debugger brings you here its probably that you forgot to call nuiEventSink<...>::SetTarget in some of you constructor code!!!
          handled = (((T*)mpTarget)->*pFunc)( pEvent );
        }
      }
    }
    mEnumerating = false;
    EmptyTrash();
    return handled;
  }

  virtual void Connect(nuiEventSource& rSource, TargetFunc pTargetFunc, void* pUser=NULL)
  {
    Link* pLink = NULL;
    mEnumerating = true;
    typename std::list<Link*>::iterator it;
    typename std::list<Link*>::iterator end = mpLinks.end();
    bool allreadyconnected = false;

    for (it = mpLinks.begin(); it!=end && !pLink; it++)
    {
      if ((*it)->mpSource == &rSource)
      {
        pLink = *it;
        allreadyconnected = true;
      }
    }

    if (!pLink)
    {
      pLink = new Link;
      pLink->mpSource = &rSource;
      pLink->mpUser = pUser;
    }

    pLink->mpTargets.push_front(pTargetFunc);

    rSource.Connect(this);

    if (!allreadyconnected)
      mpLinks.push_front(pLink);

    mEnumerating = false;
    EmptyTrash();
  }

  virtual void Disconnect(nuiEventSource& rSource)
  {
    if (mEnumerating)
    {
      mpTrash.push_back(new DisconnectOrder(&rSource,NULL));
      return;
    }
    std::list<Link*> templist;
    typename std::list<Link*>::iterator it;
    typename std::list<Link*>::iterator end = mpLinks.end();
  
    for (it=mpLinks.begin(); it != end; it++)
    {
      Link* pLink = *it;
      if (pLink)
      {
        if ((pLink)->mpSource == &rSource)
        {
          pLink->mpSource->Disconnect(this);
          templist.push_back (pLink);
        }
      }
    }

    end = templist.end();

    for (it=templist.begin(); it != end; it++)
    {
      Link* pLink = *it;
      delete pLink;
      mpLinks.remove(pLink);   
    }
  }

  virtual void Disconnect(TargetFunc pTargetFunc)
  {
    if (mEnumerating)
    {
      mpTrash.push_back(new DisconnectOrder(NULL,pTargetFunc));
      return;
    }
    std::list<Link*> templist;
    typename std::list<Link*>::iterator it;
    typename std::list<Link*>::iterator end = mpLinks.end();
    
    for (it=mpLinks.begin(); it != end; it++)
    {
      (*it)->mpTargets.remove(pTargetFunc);
      if (!(*it)->mpTargets.size())
      {
        (*it)->mpSource->Disconnect(this);
        templist.push_back(*it);
      }
    }

    end = templist.end();

    for (it=templist.begin(); it != end; it++)
    {
      delete *it;
      mpLinks.remove(*it);
    }
  }

  virtual void Disconnect(nuiEventSource& rSource,TargetFunc pTargetFunc)
  {
    if (mEnumerating)
    {
      mpTrash.push_back(new DisconnectOrder(&rSource,pTargetFunc));
      return;
    }
    std::list<Link*> templist;
    typename std::list<Link*>::iterator it;
    typename std::list<Link*>::iterator end = mpLinks.end();
    
    for (it=mpLinks.begin(); it != end; it++)
    {
      if ((*it)->mpSource == &rSource)
      {
        (*it)->mpTargets.remove(pTargetFunc);
        if (!(*it)->mpTargets.size())
        {
          (*it)->mpSource->Disconnect(this);
          templist.push_back(*it);
        }
      }
    }

    end = templist.end();

    for (it=templist.begin(); it != end; it++)
    {
      delete *it;
      mpLinks.remove(*it);
    }
  }

  virtual void EmptyTrash()
  {
    typename std::list<DisconnectOrder*>::iterator it;
    typename std::list<DisconnectOrder*>::iterator end = mpTrash.end();
    
    for (it=mpTrash.begin(); it != end; it++)
    {
      if (*it)
      {
        nuiEventSource* pSource = (*it)->mpSource;
        TargetFunc pFunc = (*it)->mpFunc;
        if (pSource && pFunc)
        {
          Disconnect(*pSource, pFunc);
        }
        else if (pSource)
        {
          Disconnect(*pSource);
        }
        else if (pFunc)
        {
          Disconnect(pFunc);
        }
        delete (*it);
      }
    }

    mpTrash.clear();
  }

protected:
  // Target handling:
  void SetTarget(T* pTarget)
  {
    mpTarget = pTarget;
  }
  T* GetTarget()
  {
    return mpTarget;
  }

  // Trash mecanism:
  bool mEnumerating;
  
  class DisconnectOrder
  {
  public:
    DisconnectOrder(nuiEventSource* pSource, TargetFunc pFunc)
    {
      mpSource = pSource;
      mpFunc = pFunc;
    }
    virtual ~DisconnectOrder()
    {
    }
    nuiEventSource* mpSource;
    TargetFunc mpFunc;
  };

  std::list <DisconnectOrder*> mpTrash;
private:
  class Link
  {
  public:
    std::list <TargetFunc> mpTargets;
    nuiEventSource* mpSource;
    void* mpUser;
  };
  T* mpTarget;
  std::list <Link*> mpLinks;

  nuiEventTarget(const nuiEventTarget& rTarget)
  {
    // There is NO way to copy a target properly. 
  }
};

/// This template is the basis of the event reception system of nui.
template <class T> class nuiEventSink : public nuiEventTarget<T>
{
public:
  nuiEventSink()
  {
    SetTarget(NULL);
  }
  virtual ~nuiEventSink()
  {
  }

  void SetTarget(T* pTarget)
  {
    nuiEventTarget<T>::SetTarget(pTarget);
  }
private:
  nuiEventSink(const nuiEventSink& rSink)
  {
    // No way to use the copy constructor. 
  }
};

/// This is a helper class that implements a simple event source with a fixed type.
template <int eventtype> class nuiSimpleEventSource : public nuiEventSource
{
public:
  nuiSimpleEventSource()
  {
  }

  virtual ~nuiSimpleEventSource()
  {
  }

  virtual bool operator() (nuiEvent* pEvent=NULL)
  {
    if (pEvent == NULL)
      pEvent=new nuiEvent(this,eventtype); 
    return SendEvent(pEvent);
  }

private:
  int mEventType;
  nuiSimpleEventSource(const nuiSimpleEventSource& rSource)
  {
    // No way to implement this correctly. 
  }
};

#endif // __nuiEvent_h__
