/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiImage_h__
#define __nuiImage_h__

#include "ngl.h"
#include "nglImage.h"
#include "nuiTexture.h"
#include "nuiWidget.h"
#include "nuiRect.h"
#include "nuiColor.h"

/// This class implements a basic image widget.
class nuiImage : public nuiWidget
{
public:
  nuiImage (nuiContainerPtr pParent,nuiTexture* pTexture); ///< Create an image from an allready existing texture object.
  nuiImage (nuiContainerPtr pParent,nglIStream* pInput, nglImageCodec* pCodec = NULL); ///< Create an image from an input stream and a codec.  If \param pCodec is NULL all codecs will be tried on the image.
  nuiImage (nuiContainerPtr pParent,const nglPath& rPath, nglImageCodec* pCodec = NULL ); ///< Create an image from a path and a codec. If \param pCodec is NULL all codecs will be tried on the image.
  nuiImage (nuiContainerPtr pParent,nglImageInfo& rInfo, bool Clone = true); ///< Create an image from an nglImageInfo structure. If \param is true then the image buffer will be cloned, otherwise it will be deleted with the nuiImage.
  nuiImage (nuiContainerPtr pParent,const nglImage& rImage); ///< Create an image by copying an existing nglImage.
  nuiImage (nuiContainerPtr pParent,nglImage* pImage, bool OwnImage); ///< Create an image from an existing nglImage. If \param OwnImage the nglImage object will be deleted with the nuiImage.
  nuiImage (nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create an image from an xml description.

  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive);

  virtual ~nuiImage();

  virtual void ForceReload(); ///< This method deletes the texture assiciated with the nuiImage thus forcing its recreation at the next rendertime.
  virtual bool Draw(nuiDrawContext* pContext);
  virtual nuiRect CalcIdealSize();

  virtual bool SetCornerColor(int corner,const nuiColor& rColor); ///< Set the colors to be blended from the corners of the image (white by default).
  virtual void SetBlendFunc(GLenum sfactor = GL_SRC_ALPHA, GLenum dfactor = GL_ONE_MINUS_SRC_ALPHA); ///< Set the blending function to use while rendering to compose the image with the background (opengl enums). The default setting is to alpha blend the image with the current background.
  virtual void GetBlendFunc(GLenum& sfactor, GLenum& dfactor); ///< Get the current blending function to use while rendering to compose the image with the background (opengl enums). The default setting is to alpha blend the image with the current background.

  virtual void SetPosition(nuiPosition mode); ///< Set the position of the image. Tiled is not supported at the moment. explicit
  virtual nuiPosition GetPosition(); ///< Get the position where the image will be drawn in its actual rectangle.
  virtual void SetUseAlpha(bool Alpha); ///< If \param Alpha is true the the rendering of the image will enable alpha blending. Alpha blending is enabled by default if an nuiImage contains an alpha channel.
  virtual bool GetUseAlpha(); ///< Return true if the image rendering uses alpha blending.
  virtual void SetFillRule(nuiPosition mode); ///< If the image's position if nuiFill, this method sets the behaviour of the object when the drawing region doesn't repects the 1:1 ratio of the image. 
  virtual nuiPosition GetFillRule(); ///< If the image's position if nuiFill, this method gets the behaviour of the object when the drawing region doesn't repects the 1:1 ratio of the image. 
  virtual nglImage* GetImage(); ///< Return a pointer to the nglImage contained in this object's nuiTexture.
  virtual nuiTexture* GetTexture(); ///< Return a pointer to the nuiTexture contained in this object.

protected:
  nuiVPosition mPosition;
  nuiVPosition mFillRule;
  nuiTexture* mpTexture;
  nuiBool mUseAlpha;
  nuiColor mpColors[4];

  GLenum mSrcBlend;
  GLenum mDstBlend;
};

#endif // __nuiImage_h__
