/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiList_h__
#define __nuiList_h__

#include "ngl.h"
#include "nglPath.h"
#include "nuiApplication.h"
#include "nuiEvent.h"
#include "nuiMouseEvent.h"
#include "nuiScrollBar.h"
#include "nuiContainer.h"

/// This class implements a simple list of elements. It has no multicolumn support, see nuiTable for multicolumn lists.
class nuiList : public nuiContainer
{
public:
  nuiList(nuiContainerPtr pParent,nuiOrientation orientation = nuiVertical);
  nuiList(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  virtual ~nuiList();

  void InitProperties();

  virtual bool Draw(nuiDrawContext* pContext);
  virtual nuiRect CalcIdealSize();
  virtual bool SetRect(const nuiRect& rRect);

  void SetOrientation(nuiOrientation orientation); /// Set the widget orientation.
  nuiOrientation GetOrientation(); ///< Get the widget orientation.
  nuiWidgetPtr GetItem(nuiSize X,nuiSize Y); ///< Return a pointer to the child pointed by X,Y or NULL.
  int32 GetItemNumber(nuiWidgetPtr pWidget); ///< Return the index of the child pointed by pWidget.
  bool GetUnselectable(); ///< Return true if the list items are unselectable by clicking on it.
  bool GetMultiSelectable(); ///< Return true if multiple list items can be selected.
  void SetUnselectable(bool set); ///< If true then the selected list items can be deselected by clicking on it.
  void SetMultiSelectable(bool set); ///< If true then multiple list items can be selected at the same time.
  uint DeselectAll(); ///< Deselect all items.
  uint SelectAll(); ///< Select all items (if the list is multiselectable only).
  uint GetSelected(nuiWidgetList& selitems); ///< Populate \param selitems with all the selected items.
  uint GetUnselected(nuiWidgetList& unselitems); ///< Populate \param unselitems with all the not selected items.
  bool Clear(bool Delete = true); ///< Delete all list items (makes sure the scroll bar is NOT deleted!! If you really want to remove/delete all children explicitely use nuiContainer::Clear()).
  bool ShowRow(uint32 number); ///< Make the given row visible by scrolling the list.  

  // Helper function to make lists of files:
  bool PopulateFiles(const nglPath& rPath); ///< Fill the list box with the list of the files contained in the given path.
  bool PopulateDirs(const nglPath& rPath);  ///< Fill the list box with the list of the directories contained in the given path.
  bool Populate(const nglPath& rPath, bool Files = true, bool Dirs = true); ///< Fill the list box with the list of the files and/or the directorie contained in the given path.

  // Other atributes:
  nuiSize GetBorderSize(); ///< Retreive the current border around the area of the enclosed widget.
  void SetBorderSize(nuiSize BorderSize); ///< Set the current border around the area of the enclosed widget.
  void SetClipChildren(bool Clip);
  bool GetClipChildren();
  void SetFixedAspectRatio(bool flag);
  bool GetFixedAspectRatio();
  void ShowScrollBar(bool show); ///< This method permits to hide and show the scrollbar (shown by default).
  bool IsScrollBarVisible(); ///< This method returns true if the scrollbar is visible.

  // Class attributes:
  static nuiSize GetDefaultBorderSize(); ///< Retreive the default border around the area of the enclosed widget of every button that will be created.
  static void SetDefaultBorderSize(nuiSize BorderSize); ///< Set the default border around the area of the enclosed widget of every button that will be created.

  // Outgoing event:
  nuiMouseClicked Clicked; ///< This event is called whenever an item is clicked.
  nuiMouseClicked DoubleClicked; ///< This event is called whenever an item is chosen, that is to say, double clicked. This can only happend in a mono-selection list.
  nuiSimpleEventSource<nuiWidgetActivated> Activated; ///< This event is called whenever an item is chosen, that is to say, double clicked. This can only happend in a mono-selection list.
  nuiSimpleEventSource<nuiWidgetSelected> SelectionChanged; ///< This event is called whenever selection of the items of the list have changed. 

  // Incomming events:
  bool KeyDown (nglKeyCode Key, nglChar Char);
  bool KeyUp (nglKeyCode Key, nglChar Char);
  bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  bool MouseMoved  (nuiSize X, nuiSize Y);

  bool Scrolled(nuiEvent* pEvent); ///< Called whenever the scroll bar is changed.

protected:
  nuiVOrientation mOrientation; /// mHorizontal is true if the containers' layout if horizontal, false if vertical.
  std::list<nuiRect>  mPositions;
  nuiFloat           mBorderSize;
  nuiBool           mClipChildren;

  // Keyboard handling variables:
  bool mMoveOnly;
  bool mKeyboardSelect;
  uint32 mCursorLine;
  bool mDisplayCursor;
  uint32 mSelectionStart;
  bool mSelection;

  nuiScrollBar*  mpScrollBar;
  static nuiSize    mDefaultBorderSize;
  nuiBool           mMultiSelectable;
  nuiBool           mUnselectable;
  nuiBool           mFixedAspectRatio;
  nuiWidgetPtr     mpLastItem;
  nuiEventSink<nuiList> mEventSink;
};

#endif // __nuiList_h__
