/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiMainWindow_h__
#define __nuiMainWindow_h__

#include "ngl.h"
#include "nglWindow.h"

#include "nuiContainer.h"
#include "nuiWindow.h"
#include "nuiEvent.h"
#include "nuiColor.h"
#include "nuiImage.h"
#include "nuiTimer.h"
#include "nuiDrawContext.h"
#include "nuiMemoryDrawContext.h"

#include <stack>

class nuiLabel;
class nuiMainWindow;
class nuiTrashElement;

enum nuiRenderer
{
  eOpenGL,
  eSoftware
};

class nuiTrashElement
{
public:
  enum ElementType
  {
    AddWidget,
    RemoveWidget,
    DeleteWidget
  };

  nuiTrashElement(ElementType type, nuiWidgetPtr pWidget, nuiContainerPtr pContainer = NULL);
  nuiTrashElement(const nuiTrashElement& rSrc);
  virtual ~nuiTrashElement();

  friend bool operator==(const nuiTrashElement& rElement1,const nuiTrashElement& rElement2);
  friend class nuiMainWindow;

protected:
  ElementType mType;
  nuiWidgetPtr mpWidget;
  nuiContainerPtr mpContainer;
};

/// This class implements the root nui object: the main window of any application.
class nuiMainWindow :  public nglWindow, public nuiContainer
{
public:
  nuiMainWindow(const nglContextInfo& rContextInfo, const nglWindowInfo& rInfo, const nglContext* pShared = NULL, const nglPath& mResPath = nglPath(ePathCurrent));
  nuiMainWindow(uint Width, uint Height, bool Fullscreen=false, const nglPath& mResPath = nglPath(ePathCurrent));
  nuiMainWindow(nuiContainerPtr pParent, nuiXMLNode* pNode, int W, int H, bool Fullscreen, const nglPath& mResPath = nglPath(ePathCurrent)); ///< Create from an XML description.

  virtual ~nuiMainWindow();

  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive);


  /* @name Inheritance from nglWindow */
  //@{
  virtual void OnPaint();
  virtual void OnResize (uint Width, uint Height);
  virtual void OnCreation();
  virtual void OnDestruction();
  virtual void OnActivation();
  virtual void OnDesactivation();
  virtual void OnClose();
  virtual void OnState(nglWindowState State);
  virtual void OnKeyDown (nglKeyCode Key, nglChar Char);
  virtual void OnKeyUp       (nglKeyCode Key, nglChar Char);
  virtual void OnMouseClick  (int X, int Y, nglMouseFlags Button);
  virtual void OnMouseUnclick(int X, int Y, nglMouseFlags Button);
  virtual void OnMouseMove   (int X, int Y);
  //@}

  /* @name Background methods*/
  //@{
  void SetBackgroundColor(const nuiColor& rColor); ///< Set unique bg color.
  //@}

  /* @name Inherited from nuiWidget */
  //@{
  bool Draw(class nuiDrawContext *);
  void Invalidate(bool LayoutChanged = false, bool TellChildren=false);
  nuiDrawContext* GetDrawContext();
  //@}

  /* @name Popup windows management */
  //@{
  // The Active window receives all the keyboard events and dipatches them to its widgets. It also is the top level window.
  nuiWindow* GetActiveWindow(); ///< Return the current active popup window.
  bool ActivateWindow(nuiWindow* pWindow); ///< Change the current active popup window.
  bool IsParentActive(); ///< Allways true for the mainwindow!
  bool AddWindow(nuiWindow* pWindow); ///< Add a popup window to the list.
  bool DelWindow(nuiWindow* pWindow, bool Delete=false); ///< Remove a popup window from the list. If Delete==true the object will also be deleted.
  uint GetWindowCount(); ///< Return the number of popup windows contained in the main window.
  nuiWindow* GetWindow(uint index); ///< Return the window object with the given index rank in the list.
  nuiWindow* GetWindow(nuiSize X, nuiSize Y); ///< Return the window object directly under the (X,Y) pixel.
  //@}

  virtual nuiWidgetPtr GetChild(nuiSize X, nuiSize Y); ///< Reimplemented from nuiContainer::GetChild

  /* @name Mouse & keyboard focus management */
  //@{
  virtual bool Grab(nuiWidgetPtr pWidget); ///< Redirect all mouse event to this object.
  virtual bool Ungrab(); ///< Stop redirecting all mouse event to the last grab object.
  virtual nuiWidgetPtr GetGrab(); ///< Returns the object that currently has the mouse focus.

  virtual bool SetFocus(nuiWidgetPtr pWidget); ///< Redirect all keyboard events to this object.
  virtual nuiWidgetPtr GetFocus(); ///< Returns the object that currently has the keyboard focus.
 //@}

  /* @name Trash management */
  //@{
  virtual void FillTrash(); ///< Marks the start of a trash filling period: we collect all the movement going on in the widget tree.
  virtual bool IsTrashFilling(); ///< Are we collectiong change in the widget tree right now?
  virtual void EmptyTrash(); ///< Empties the object trash: all the objects that have been placed in the trash (with the Trash() method) will be delete an removed from the trash. This method should be called after processing any event (comming from nglWindow).
  virtual void AdviseObjectDeath(nuiWidgetPtr pWidget); ///< Any nuiWidget will call this method when dying. Don't forget to call this version in any overriden AdviseObjectDeath you make.
  virtual void Trash(nuiWidgetPtr pWidget); ///< Ask for the destruction of this object as soon as possible.
  virtual void AddToParent(nuiWidgetPtr pWidget,nuiContainerPtr pNewParent); ///< Ask for the addition of this object to his parent as soon as possible.
  virtual void RemoveFromParent(nuiWidgetPtr pWidget,nuiContainerPtr pParent); ///< Ask for the removal of this object from its parent as soon as possible.
  //@}

  virtual void SetClipChildren(bool set); ///< Enables or disables automatic children clipping. If set to true all children's drawing will be restrained to their rectangle. 
  virtual bool GetClipChildren(); ///< Returns children clipping status. 
  virtual void SetStaticChildren(bool Set); ///< Change the widget layout policy of the main window. If set to true all the children of the main window will be freely positionable in the window, otherwise they will be given the full client size of the window. 
  virtual bool GetStaticChildren(); ///< Return the children layout policy. See SetStaticChildren(). 

  /** @name Generic tooltips */
  //@{
  virtual bool ActivateToolTip(nuiWidgetPtr pWidget); ///< nuiMainWindow override the default tooltip mecanism to actually display them.
  virtual bool ReleaseToolTip(nuiWidgetPtr pWidget); ///< Remove the current tooltip for the given widget. 
  virtual bool ToolTipOn(nuiEvent* pEvent);
  virtual bool ToolTipOff(nuiEvent* pEvent);
  //@}

  /** @name Debugging helper functions */
  //@{
  void DBG_DisplayMouseOverInfo(); ///< Displays a tool tip near the mouse with informations on the object currently pointed to by the mouse.
  void DBG_SetMouseOverInfo(bool set);
  bool DBG_GetMouseOverInfo();
  void DBG_DisplayMouseOverObject(); ///< Displays a frame over the object currently pointed to by the mouse.
  void DBG_SetMouseOverObject(bool set);
  bool DBG_GetMouseOverObject();
  //@}

  /** @name Frame rate limitation */
  //@{
  void  SetFrameRateLimit(float fps); ///< Change the maximum framerate to use for this main window (in number of frames per second). fps should be less than or equal to 0 if you want no limit. 
  float GetFrameRateLimit(); ///< Return the maximum framerate to use for this main window (in number of frames per second). If it returns 0 there is no limit. 
  //@}

  /** @name Renderer choice: */
  //@{
  static void SetRenderer(nuiRenderer Renderer = eOpenGL);
  //@}

protected:
  nuiBool mBitmapBackGround;
  nuiBool mSolidBackGround;

  nuiColor mBgColor; // One color for each corner

//  nuiDrawContext mDrawContext;
//  nuiMemoryDrawContext mDrawContext;
  nuiDrawContext* mpDrawContext;

  nuiWidgetPtr mpGrab;
  nuiWidgetPtr mpFocus;
  nuiWidgetPtr mpUnderMouse;

  std::list<nuiWindowPtr> mpWindows;
  nuiBool mIteratingWindows;
  std::list<nuiWindowPtr> mpPreWindows;
  std::list<nuiWidgetPtr> mpGrabStack;

  nuiBool mClipChildren;
  nuiBool mStaticChildren;

  // ToolTips:
  nuiTimer mToolTipTimerOn;
  nuiTimer mToolTipTimerOff;
  nglTime mToolTipDelayOn;
  nglTime mToolTipDelayOff;
  nuiEventSink<nuiMainWindow> mMainWinSink;
  bool mDisplayToolTip;
  nuiWidgetPtr mpToolTipSource;
  nuiLabel* mpToolTipLabel;
  nuiLabel* mpInfoLabel;

  float mMaxFPS;
  nglTime mLastRendering;
  nuiTimer* mpInvalidateTimer;
  bool InvalidateTimer(nuiEvent* pEvent);
  /* @name Trash management */
  //@{
private:
  bool mFillTrash;
  std::list<nuiTrashElement> mpTrash;
  
  //@}

private:
  /* @name Debugging */
  //@{
  nuiBool mDisplayMouseOverInfo;
  nuiBool mDisplayMouseOverObject;
  //@}

  static nuiRenderer mRenderer;
};


bool operator==(const nuiTrashElement& rElement1,const nuiTrashElement& rElement2);

#endif // __nuiMainWindow_h__
