/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiMemoryDrawContext_h__
#define __nuiMemoryDrawContext_h__

#include "ngl.h"
#include "nuiDrawContext.h"

//#define NO_SOFT_BLENDING

class nuiMemoryDrawContext : public nuiDrawContext
{
public:
  nuiMemoryDrawContext(const nglPath& rResPath, const nuiRect& rRect);
  virtual ~nuiMemoryDrawContext();

  virtual void PushClipping();
  virtual void PopClipping();
  virtual bool Clip(nuiWidgetPtr pWidget, const nuiRect& rLocalRect, bool IgnorePosition = true); ///< This method restricts all the rendering operation to the given local rectangle.
  virtual bool Clip(nuiWidgetPtr pWidget); ///< This method restricts all the rendering operation to the client rectangle.
  virtual bool SetClipping(bool set); ///< Enables or disable clipping.

  virtual void SetSize(int w,int h); ///< Set the size of the drawing surface (for clipping calculus).

  /* @name Rendering helper methods */
  //@{

//  virtual void DrawShape(nuiShape* pShape, nuiShapeMode Mode, ColorMode UseColors = eContextColors, TextureMode UseTexture = eNoTexture); ///< Call StrokeShape and FillShape.
  
  // Fast and simple drawings:
  virtual void DrawRect(const nuiRect& Rect, nuiShapeMode Mode);
  virtual void DrawLine(const nglVector2f& rOrigin, const nglVector2f& rDestination);
  virtual void DrawLine(float x1, float y1, float x2, float y2);
  virtual void Clear();
  //@}

  /** @name Matrix operations: */
  //@{
  virtual void PushMatrix();
  virtual void PopMatrix();
  virtual void LoadMatrix(const nglMatrixf& Matrix);
  virtual void MultMatrix(const nglMatrixf& Matrix);
  virtual void Translate(const nglVector2f& Vector);
  virtual void Translate(nuiSize X, nuiSize Y);
  virtual void Scale(nuiSize X, nuiSize Y);
  virtual void LoadIdentity();
  //@}

  /** @name Fonts and text handling: */
  //@{
  virtual void SetFont(nuiTheme::FontStyle Style, nglFont1* pFont, bool Delete=true); ///< Changes the font used for the given style. If Delete==true the old one will be deleted.
  virtual nglFont1* GetFont(nuiTheme::FontStyle Style); ///< Retreive the font used for the given style.
  virtual bool GetFontInfo(nuiTheme::FontStyle FontStyle, nglFontInfo1& rFontInfo);
  virtual int GetTextSize (nuiTheme::FontStyle FontStyle, float& rX, float& rY, const nglChar* pText); ///< Calculate the bounding of the string in texels and returns it in X & Y.
  virtual int GetTextPos (nuiTheme::FontStyle FontStyle, float X, const nglChar* pText); ///< Calculate the bounding of the char pos in the given text where the pixel (X,?) lies and returns it.

  virtual void SetTextColor(const nuiColor& rColor, nuiTheme::FontStyle Style = nuiTheme::Default);
  virtual void DrawText(nuiSize x, nuiSize y, const nglString& rString, nuiTheme::FontStyle Style = nuiTheme::Default); /// Draw text at the given coordinates and the given font style.
  //@}

  bool SetTextureCurrent ( nuiTexture* pTex , int num = 0);


  void* GetImageBuffer() const
  {
    return mpImage;
  }

  virtual void Save(const nglPath& rPath, const nglString& rImageType); // Save the contents of the context's color buffer to a file.
  virtual void Display(nglWindow* pWindow); // Display the contents of the color buffer on a window.
  virtual void DrawImage(const nuiRect& rDest, const nuiRect& rSource);
  virtual void DrawShade (const nuiRect& rSourceRect, const nuiRect& rShadeRect);
  virtual void BlurRect(const nuiRect& rRect, uint Strength);

protected:
  static bool StaticGlyphHandler(FT_Bitmap* pBitmap, float Left, float Top, void* pUser);
  bool HandleGlyph(FT_Bitmap* pBitmap, float Left, float Top);

  virtual void ActivateClipper();

//  virtual void InternalTessEnd();
//  virtual void InternalTessCombine(GLdouble coords[3], void *vertex_data[4], GLfloat weight[4], void **outData);
//  virtual void InternalTessError(GLenum ErrNo);

//  virtual void TesselateShape(nuiShape* pShape, nuiShapeMode Mode, ColorMode UseColors = eContextColors, TextureMode UseTexture = eNoTexture);

  std::stack<nglMatrixf> mMatrixStack;
  std::map<nuiTheme::FontStyle,nuiColor> mFontColors;
  nuiColor mFontColor;

  class Color
  {
  public:
#ifdef __CARBON__
    uint32 mR : 8, 
           mG : 8, 
           mB : 8, 
           mA : 8;
#else
    uint32 mB : 8, 
           mG : 8, 
           mR : 8, 
           mA : 8;
#endif
    inline int GetInt() const
    {
      return *(int*)this;
    }

    inline void BlendWith(const Color& color)
    {
//      uint R,G,B;
#ifndef NO_SOFT_BLENDING
  #if 1
      mR += (((short)color.mR-(short)mR) * (short)color.mA) >> 8;
      mG += (((short)color.mG-(short)mG) * (short)color.mA) >> 8;
      mB += (((short)color.mB-(short)mB) * (short)color.mA) >> 8;
  #else
      int tmp;
      tmp  = ((short)color.mR-(short)mR);
      tmp |= ((short)color.mB-(short)mB)<<16;
      tmp &= 0xFF00FF;
      tmp *= (int)color.mA;
      tmp &= 0xFF00FF00;
      tmp |= ((((short)color.mG-(short)mG) * (short)color.mA) & 0XFF00)<<8;

      *((uint*)this) += tmp;
  #endif
#else
      *this = color;
#endif
    }

    inline void FastBlendWith(const Color& color)
    {
//      uint R,G,B;
#ifndef NO_SOFT_BLENDING

#if 0
      mR = (mR + color.mR) >> 1;
      mG = (mG + color.mG) >> 1;
      mB = (mB + color.mB) >> 1;
#else
      mR >>= 1;
      mR += color.mR >> 1;
      mG >>= 1;
      mG += color.mG >> 1;
      mB >>= 1;
      mB += color.mB >> 1;
#endif

#else
      *this = color;
#endif
    }

    inline Color& operator =(const nuiColor& rColor)
    {
      mR = (uint8)(rColor.GetRed() * 255.0f);
      mG = (uint8)(rColor.GetGreen() * 255.0f);
      mB = (uint8)(rColor.GetBlue() * 255.0f);
      mA = (uint8)(rColor.GetAlpha() * 255.0f);
      return *this;
    }
  };

  Color* mpImage;
  Color* mpLineBuffer;

  // Without texturing:
  virtual void RasterLine(Color* pScreen, uint size, const Color& rColor); // Fill a line with the given color
  virtual void RasterLineA(Color* pScreen, uint size, const Color& rColor); // Fill a line with the given color and alpha blending.
  virtual void RasterLine(Color* pScreen, uint size, const Color& rSColor, const Color& rDColor); // Fill a line interpolating with the given colors.
  virtual void RasterLineA(Color* pScreen, uint size, const Color& rSColor, const Color& rDColor); // Fill a line interpolating with the given colors using alpha blending.

  virtual void StretchRaster16(char* pDst, char* pSrc, uint size1, uint size2);
  virtual void StretchRaster24(char* pDst, char* pSrc, uint size1, uint size2);
  virtual void StretchRaster32(char* pDst, char* pSrc, uint size1, uint size2);
  virtual void CopyRaster16(char* pDst, char* pSrc, uint size1, uint size2);
  virtual void CopyRaster24(char* pDst, char* pSrc, uint size1, uint size2);
  virtual void CopyRaster32(char* pDst, char* pSrc, uint size1, uint size2);

  virtual void RasterBlend( nuiMemoryDrawContext::Color* pDst, nuiMemoryDrawContext::Color* pSrc, uint count);

  typedef void (nuiMemoryDrawContext::*CopyRasterFunc)(char* pDst, char* pSrc, uint size1, uint size2);
  typedef void (nuiMemoryDrawContext::*DrawPixelFunc)(Color* pScreen, const Color& rColor);
  void SetPixel(Color* pScreen, const Color& rColor);
  void BlendPixel(Color* pScreen, const Color& rColor);
  void BresenhamLine(int x0, int y0, int x1, int y1, DrawPixelFunc _DrawPixel, const Color& rColor);

  // These two arrays will contain the triangle borders precalculated for the rasteriser.
  int* mpTriangleMin;
  int* mpTriangleMax;
  Color* mpColorMin;
  Color* mpColorMax;
  int mTriRasterStart;
  int mTriRasterEnd;
  void DrawTriangle(TempVert* pP1, TempVert* pP2, TempVert* pP3);
  void BresenhamTriangleLine(TempVert* pOrigin, TempVert* pDestination);
  void CalcTriangleBorder(TempVert* pOrigin, TempVert* pDestination);

  void DrawCachedShape(std::list<nuiShape::CacheElement*>* pElements, nuiShapeMode Mode, ColorMode UseColors, TextureMode UseTexture);
  void DrawTriangle(nuiShape::CacheElement::Vertex* pP1, nuiShape::CacheElement::Vertex* pP2, nuiShape::CacheElement::Vertex* pP3);
};

nuiMemoryDrawContext* CreateMemoryDrawContext(const nglPath& rResPath, const nuiRect& rRect); ///< Use this method if you want to get the best possible MemoryDrawContext for your architecture.

#endif //__nuiMemoryDrawContext_h__

