/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiObject_h__
#define __nuiObject_h__

#include "ngl.h"
#include "nglString.h"
#include "nuiApplication.h"
#include "nuiFlags.h"
#include "nuiTimer.h"
#include "nuiProperty.h"
#include <list>
#include <map>

class nuiEventSource;

class nuiXMLNode;

enum nuiSerializeMode
{
  eDontSaveNode, ///< Do not serialize this node and any of its children if any.
  eSaveNode, ///< Serialize this node and all its children (unless a child is explicitely set to be skipped or not saved.)
  eSkipNode  ///< Skip this node's serilization and only serialize its children if any.
};

/// This is the root nui class. It implements all the basic widget management system.
class nuiObject
{
public:
  /** @name Life */
  //@{
  nuiObject(); ///< Create an nuiObject and give him his parent.
  nuiObject(nuiXMLNode* pNode); ///< Create an nuiObject from an XML description.
  virtual ~nuiObject(); 
  //@}

  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pNode, bool Recursive = true); ///< This method saves the widget state in the given XML Node. If @param Recursive == true (default) then the children nodes will be serialized too. Returns true on succes. 
  virtual void SetSerializeMode (nuiSerializeMode eMode); ///< Set the serialization mode for this object (see nuiSerializeMode enum documentation).
  virtual nuiSerializeMode GetSerializeMode (); ///< Get the serialization mode for this object (see nuiSerializeMode enum documentation).
  //@}

  /** @name Properties system */
  //@{
  virtual void SetProperty(const nglString& rName, const nglString& rValue); ///< Add or change a property of the object.
  virtual nglString GetProperty(const nglString& rName); ///< Return the property value corresponding to the given property name. If the object doesn't have the property the returned string is empty.
  virtual bool GetProperty(const nglString& rName, nuiProperty*& rpProperty); ///< Put the property object corresponding to the given property name in the rpProperty variable.
  virtual nuiEventSource* GetPropertyEvent(const nglString& rName); ///< Return the property event target "Changed". 
  virtual bool HasProperty(const nglString& rName); ///< Return true if the object contains the property.
  virtual bool GetProperties(std::list<nglString>& rPropertyNames); ///< Populate @param rPropertyNames with the name of the properties of the object. 
  virtual bool IsPropertyExportable(const nglString& rName); ///< Returns true is the property is flagged as exportable (i.e. serializable). 
  virtual bool IsPropertyReadOnly(const nglString& rName); ///< Returns true if one can only read this property. 
  virtual nuiValueType GetPropertyType(const nglString& rName); ///< Returns the type of the given property. 

  virtual const nuiPropertyInfo* GetPropertyInfo(uint index); ///< Returns a pointer to the property info that have the given index. It must also take care of the properties it inherits from is ancestors by calling the equivalent method on them if it doesn't implement the property explicitely. If the index is too large the function returns NULL.
  virtual const nuiPropertyInfo* GetPropertyInfo(const nglString& rName); ///< Returns a pointer to the property info that have the given name. It must also take care of the properties it inherits from is ancestors by calling the equivalent method on them if it doesn't implement the property explicitely. 
  /*!<
    Returns a static pointer to the property info that have the given name or NULL if there is no property info of this name. 
    This method calls GetPropertyInfo(int).

    \b WARNING! \b
    When you override this method you MUST add this code to your implementation:
    BASECLASS if the name of the class from which you inherit and YOURCLASS if the name of the new class.
\code
const nuiPropertyInfo* YOURCLASS::GetPropertyInfo(uint index)
{
  int count=(sizeof(gpObjectPropertiesInfos)/sizeof(nuiPropertyInfo));
  if (index < count)
    return &gpObjectPropertiesInfos[index];

  const nuiPropertyInfo* pInfo= BASECLASS::GetPropertyInfo(index - count);
  if (pInfo)
    return GetPropertyInfo(nglString(pInfo->mpName));

  return NULL;
}
\endcode
  */
  //@}

protected:
  void InitProperties(); ///< Take care of the property bindings.

  typedef std::map<nglString,nuiProperty,nglString::LessFunctor> nuiPropertyMap;
  nuiPropertyMap mProperties;

  nuiSerializeMode mSerializeMode;
};

//typedef nuiSmartPtr<nuiObject> nuiObjectPtr;
typedef nuiObject* nuiObjectPtr;
typedef std::list<nuiObjectPtr> nuiObjectList;

#define NUI_PROP(x,y,z) {_T(x),_T(y),_T(z),true,true} ///< Define a property as exportable and writable by the user.
#define NUI_PROPEX(x,y,z,exportable,writable) {_T(x),_T(y),_T(z),exportable,writable} ///< Define a property and let the programmer choose the acces rights to it.

#endif // __nuiObject_h__
