/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiPoint_h__
#define __nuiPoint_h__

#include "ngl.h"
#include "nglVector.h"
#include "nuiColor.h"
#include "nuiXML.h"
#include <vector>

// These are the meta data standard positions:
#define NUI_POINT_TEXCOORD_X 0
#define NUI_POINT_TEXCOORD_Y 1
#define NUI_POINT_COLOR_R 2
#define NUI_POINT_COLOR_G 3
#define NUI_POINT_COLOR_B 4
#define NUI_POINT_COLOR_A 5

class nuiPoint: public nglVectorf
{
public:
  nuiPoint() : nglVectorf()
  { 
  }

  nuiPoint(nuiXMLNode* pNode) : nglVectorf()
  { 
    if (pNode->GetName() != _T("nuiPoint"))
      return;

    Elt[0] = nuiGetVal(pNode,_T("X"), 0.0f);
    Elt[1] = nuiGetVal(pNode,_T("Y"), 0.0f);
    Elt[2] = nuiGetVal(pNode,_T("Z"), 0.0f);

    uint i = 0;
    std::list<nuiXMLNode*> children;
    children = pNode->GetChildren();

    std::list<nuiXMLNode*>::iterator it;
    std::list<nuiXMLNode*>::iterator end = children.end();

    std::vector<float> metadata;
    for (it = children.begin(); it != end; it++)
    {
      nuiXMLNode* pChild = *it;
      if (pChild)
      {
        if (pChild->GetName() == _T("MetaData"))
        {
          metadata.resize(i+1);
          metadata[i]=nuiGetVal(pChild,"Value",0.0f);
          i++;
        }
      }
    }
    SetMetaData(metadata);
  }

  /** @name Serialisation */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode) const ///< This method saves the state in the given XML Node.
  {
    nuiXMLNode* pNode = NULL;
    if (pParentNode)
      pNode = new nuiXMLNode(_T("nuiPoint"),pParentNode);
    else
      pNode = new nuiXML(_T("nuiPoint"));

//    pNode->SetAttribute(_T("Name"),GetProperty(_T("Name")));
    pNode->SetAttribute(_T("X"),Elt[0]);
    pNode->SetAttribute(_T("Y"),Elt[1]);
    pNode->SetAttribute(_T("Z"),Elt[2]);

    uint i;
    uint size = (uint)mMetaData.size();
    for (i=0; i < size; i++)
    {
      nuiXMLNode* pMetaNode = new nuiXMLNode(_T("MetaData"), pNode);
      pMetaNode->SetAttribute(_T("Value"),mMetaData[i]);
    }

    return pNode;
  }
  //@}

  /// Initialize with values
  nuiPoint(float X, float Y, float Z, float W = 1.f) : nglVectorf(X,Y,Z,W)
  {
  }

  /// Initialize with a two dimensional vector. Third element is zero, fourth is one.
  nuiPoint(const nglVector2f& rVector) : nglVectorf(rVector)
  { 
  }

  /// Initialize with a two dimensional vector. Fourth element is one.
  nuiPoint(const nglVector3f& rVector)  : nglVectorf(rVector)
  { 
  }

  /// Initialize with a four dimensional vector.
  nuiPoint(const nglVectorf& rVector)  : nglVectorf(rVector)
  { 
  }

  /// Copy constructor
  nuiPoint(const nuiPoint& rPoint)  : nglVectorf(rPoint)
  {
    rPoint.GetMetaData(mMetaData);
  }

  nuiPoint& operator=(const nuiPoint& rPoint)
  {
    *((nglVectorf*)this) = (nglVectorf)rPoint;
    rPoint.GetMetaData(mMetaData);
    return *this;
  }

  /// Destructor
  virtual ~nuiPoint() ///< Destructor.
  {
  }

  /** @name Meta data management: */
  //@{
  void SetMetaData(const std::vector<float>& Data)
  {
    mMetaData.assign(Data.begin(),Data.end());
  }

  void GetMetaData(std::vector<float>& Data) const
  {
    Data.assign(mMetaData.begin(),mMetaData.end());
  }
  //@}

  /** @name Meta data helper methods: */
  //@{
  void SetColor(const nuiColor& Color)
  {
    if (mMetaData.size() < NUI_POINT_COLOR_A+1)
      mMetaData.resize(NUI_POINT_COLOR_A+1);
    mMetaData[NUI_POINT_COLOR_R] = Color.GetRed();
    mMetaData[NUI_POINT_COLOR_G] = Color.GetGreen();
    mMetaData[NUI_POINT_COLOR_B] = Color.GetBlue();
    mMetaData[NUI_POINT_COLOR_A] = Color.GetAlpha();
  }

  nuiColor GetColor() const
  {
    if (mMetaData.size() < NUI_POINT_COLOR_A+1)
    {
      // If there was no color yet fill with black:
      mMetaData.resize(NUI_POINT_COLOR_A+1);
      mMetaData[NUI_POINT_COLOR_A] = 1;
    }
    return nuiColor(mMetaData[NUI_POINT_COLOR_R],
                    mMetaData[NUI_POINT_COLOR_G],
                    mMetaData[NUI_POINT_COLOR_B],
                    mMetaData[NUI_POINT_COLOR_A]);
  }

  void SetTexCoord(const nglVector2f& TexCoord)
  {
    if (mMetaData.size() < NUI_POINT_TEXCOORD_Y+1)
      mMetaData.resize(NUI_POINT_TEXCOORD_Y+1);
    mMetaData[NUI_POINT_TEXCOORD_X] = TexCoord.Elt[0];
    mMetaData[NUI_POINT_TEXCOORD_Y] = TexCoord.Elt[1];
  }

  nglVector2f GetTexCoord() const
  {
    if (mMetaData.size() < NUI_POINT_TEXCOORD_Y+1)
      mMetaData.resize(NUI_POINT_TEXCOORD_Y+1);
    return nglVector2f(mMetaData[NUI_POINT_TEXCOORD_X], mMetaData[NUI_POINT_TEXCOORD_Y]);
  }
  //@}
protected:
  mutable std::vector<float> mMetaData; ///< This contains the metadata: texture coordinates, normals, colors, etc...
};

#endif // __nuiPoint_h__
