/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiProperty_h__
#define __nuiProperty_h__

#include "nuiFlags.h"
#include "nuiEventRegistry.h"
#include "nuiEvent.h"

//#define USE_NUIVALUETYPES

typedef struct nuiPropertyInfo
{
  nglChar* mpName; ///< The property name.
  nglChar* mpDefaultValue; ///< The property default value. 
  nglChar* mpInfo; ///< An optional string describing the property's meaning.
  bool mExportable; ///< True if the property is exported in xml
  bool mWritable; ///< True if the property can be written, false if read only.
} nuiPropertyInfo;

enum nuiValueType 
{
  nuiTypeNone = 0,
  
  nuiTypeValue, ///< An object that inherits from nuiValue. 

  nuiTypeBool,

  nuiTypeInt64,
  nuiTypeUInt64,
  nuiTypeInt32,
  nuiTypeUInt32,
  nuiTypeInt16,
  nuiTypeUInt16,
  nuiTypeInt8,
  nuiTypeUInt8,

  nuiTypeString,

  nuiTypeFloat,
  nuiTypeDouble,

  nuiTypeDirection,
  nuiTypePosition,
  nuiTypeOrientation,
  nuiTypeState,
  nuiTypeColor
};

class nuiValue
{
public:
  nuiValue()
  {
    mValueType = nuiTypeNone;
  }

  virtual ~nuiValue()
  {
  }

  nuiValueType GetType()
  {
    return mValueType;
  }

  virtual nglString GetValue()
  {
    return nglString::Empty;
  }

  virtual bool SetValue(const nglString& value)
  {
    return false;
  }

  // Outgoing events:
  nuiSimpleEventSource<nuiValueChanged> Changed;

protected:
  nuiValueType mValueType;
private:
  nuiValue(const nuiValue& rVal)
  {
  }


};

template<class T> class nuiValueTmpl : public nuiValue
{
public:
  nuiValueTmpl()
  { 
  }

  nuiValueTmpl(const T& val)
  { 
    mValue = val;
  }

  nuiValueTmpl(const nuiValueTmpl<T>& val)
  { 
    mValue = val.mValue;
  }

  virtual ~nuiValueTmpl()
  {
  }

  virtual nglString GetValue()
  {
    return nuiValue::GetValue();
  }

  virtual bool SetValue(const nglString& value)
  {
    return nuiValue::SetValue(value);
  }

  operator T& () 
  { 
    return mValue; 
  }

  T operator=(const T& val) 
  { 
    return mValue = val; 
  }

  T operator=(const nuiValueTmpl<T>& val) 
  { 
    return mValue = val.mValue; 
  }

  bool operator==(const T& val) 
  { 
    return mValue == val; 
  }

  bool operator==(const nuiValueTmpl<T>& val) 
  { 
    return mValue == val.mValue; 
  }

  bool operator!() 
  { 
    return !mValue; 
  }

  bool operator||(const T& val) 
  { 
    return mValue || val; 
  }

  T operator|(const T& val) 
  { 
    return mValue | val; 
  }

  bool operator&&(const T& val) 
  { 
    return mValue && val; 
  }

  T operator&(const T& val) 
  { 
    return mValue & val; 
  }

protected:
  T mValue;
};

class nuiProperty
{
public:
  nuiProperty();
  nuiProperty(const nuiProperty& rProperty);
  virtual ~nuiProperty();
  
  bool SetValue(const nglString& Value); // set the value of the property from the given string. 
  nglString GetValue();   // return the value of the property as a string. 
  nuiValueType GetType(); // return the type of the bound value of nuiTypeNone if no variable is bound.

  operator nglString();

  bool Bind(nuiValue* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a nuiValue variable to this property.

  bool Bind(bool* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a bool variable to this property.

  bool Bind(int8* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(int16* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(int32* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(int64* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.

  bool Bind(uint8* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(uint16* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(uint32* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(uint64* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.

  bool Bind(float* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(double* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.

  bool Bind(nglString* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.

  bool Bind(nuiPosition* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(nuiDirection* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(nuiOrientation* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.
  bool Bind(nuiState* pValue, bool Exportable=false, bool ReadOnly=false); ///< Bind a variable to this property.

  bool IsExportable()  ///< Return true if this property can be serialized of exported. 
  {
    return mExportable;
  }

  void SetExportable(bool set)
  {
    mExportable = set;
  }

  bool IsReadOnly() ///< Return true if this property can be changed.
  {
    return mReadOnly;
  }

  void SetReadOnly(bool set)
  {
    mReadOnly = set;
  }

  nuiSimpleEventSource<nuiValueChanged> Changed; ///< This event is fired whenever the property is changed. Beware, only properly bound nuiValues can fire events when the source variable is changed!

protected:
  bool ValueChanged(nuiEvent* pEvent);
  nglString mValue;
  nuiValue* mpValue; ///< If this is NULL then use mVariable or mValue as the value.
  void* mpVariable; ///< If this is NULL then use mValue as the value. If it is not null then refer to mType to get the actual type pointed to. 
  nuiValueType mType; ///< Contains the type of the property.
  bool mExportable;
  bool mReadOnly;
  nuiEventSink<nuiProperty> mPropertySink;
};

#ifndef USE_NUIVALUETYPES
typedef bool nuiBool;

typedef int8 nuiInt8;
typedef int16 nuiInt16;
typedef int32 nuiInt32;
typedef int64 nuiInt64;

typedef uint8 nuiUInt8;
typedef uint16 nuiUInt16;
typedef uint32 nuiUInt32;
typedef uint64 nuiUInt64;

typedef float nuiFloat;
typedef double nuiDouble;

typedef nglString nuiString;

typedef nuiPosition nuiVPosition;
typedef nuiDirection nuiVDirection;
typedef nuiOrientation nuiVOrientation;
typedef nuiState nuiVState;

#else
///////////////////////////////////////////////////
// nuiValue specialisations:
///////////////////////////////////////////////////

// Special types for handling nui properties inside widgets:
typedef nuiValueTmpl<bool> nuiBool;

typedef nuiValueTmpl<int8> nuiInt8;
typedef nuiValueTmpl<int16> nuiInt16;
typedef nuiValueTmpl<int32> nuiInt32;
typedef nuiValueTmpl<int64> nuiInt64;

typedef nuiValueTmpl<uint8> nuiUInt8;
typedef nuiValueTmpl<uint16> nuiUInt16;
typedef nuiValueTmpl<uint32> nuiUInt32;
typedef nuiValueTmpl<uint64> nuiUInt64;

typedef nuiValueTmpl<float> nuiFloat;
typedef nuiValueTmpl<double> nuiDouble;

typedef nuiValueTmpl<nglString> nuiString;

typedef nuiValueTmpl<nuiPosition> nuiVPosition;
typedef nuiValueTmpl<nuiDirection> nuiVDirection;
typedef nuiValueTmpl<nuiOrientation> nuiVOrientation;
typedef nuiValueTmpl<nuiState> nuiVState;

/*
 * nuiValue templates
 */

nglString nuiBool::GetValue()
{
  return mValue?_T("true"):_T("false");
}

bool nuiBool::SetValue(const nglString& rValue)
{
  if ( _T("true") == rValue)
    mValue = true;
  else if ( _T("false") == rValue)
    mValue = false;
  else
    return false;

  Changed();
  return true;
}


// Signed integer values:
nglString nuiInt8::GetValue()
{
  nglString str;
  str.CFormat("%d",mValue);
  return str;
}

bool nuiInt8::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCInt();
  Changed();
  return true;
}

nglString nuiInt16::GetValue()
{
  nglString str;
  str.CFormat("%d",mValue);
  return str;
}

bool nuiInt16::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCInt();
  Changed();
  return true;
}

nglString nuiInt32::GetValue()
{
  nglString str;
  str.CFormat("%d",mValue);
  return str;
}

bool nuiInt32::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCInt();
  Changed();
  return true;
}

nglString nuiInt64::GetValue()
{
  nglString str;
  str.CFormat("%d",mValue);
  return str;
}

bool nuiInt64::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCInt();
  Changed();
  return true;
}


// Unsigned Integers values:
nglString nuiUInt8::GetValue()
{
  nglString str;
  str.CFormat("%u",mValue);
  return str;
}

bool nuiUInt8::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCInt();
  Changed();
  return true;
}

nglString nuiUInt16::GetValue()
{
  nglString str;
  str.CFormat("%u",mValue);
  return str;
}

bool nuiUInt16::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCInt();
  Changed();
  return true;
}

nglString nuiUInt32::GetValue()
{
  nglString str;
  str.CFormat("%u",mValue);
  return str;
}

bool nuiUInt32::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCInt();
  Changed();
  return true;
}

nglString nuiUInt64::GetValue()
{
  nglString str;
  str.CFormat("%u",mValue);
  return str;
}

bool nuiUInt64::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCInt();
  Changed();
  return true;
}


// Floating point values:
nglString nuiFloat::GetValue()
{
  nglString str;
  str.CFormat("%f",mValue);
  return str;
}

bool nuiFloat::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCFloat();
  Changed();
  return true;
}

nglString nuiDouble::GetValue()
{
  nglString str;
  str.CFormat("%f",mValue);
  return str;
}

bool nuiDouble::SetValue(const nglString& rValue)
{
  mValue = rValue.GetCDouble();
  Changed();
  return true;
}

// Strings:
nglString nuiString::GetValue()
{
  return mValue;
}

bool nuiString::SetValue(const nglString& rValue)
{
  mValue = rValue;
  Changed();
  return true;
}

// nuiVPosition:
nglString nuiVPosition::GetValue()
{
  return nuiGetPosition(mValue);
}

bool nuiVPosition::SetValue(const nglString& rValue)
{
  mValue = nuiGetPosition(rValue);
  Changed();
  return true;
}

// nuiVDirection
nglString nuiVDirection::GetValue()
{
  return nuiGetDirection(mValue);
}

bool nuiVDirection::SetValue(const nglString& rValue)
{
  mValue = nuiGetDirection(rValue);
  Changed();
  return true;
}

// nuiVOrientation
nglString nuiVOrientation::GetValue()
{
  return nuiGetOrientation(mValue);
}

bool nuiVOrientation::SetValue(const nglString& rValue)
{
  mValue = nuiGetOrientation(rValue);
  Changed();
  return true;
}

// nuiVState:
nglString nuiVState::GetValue()
{
  return nuiGetState(mValue);
}

bool nuiVState::SetValue(const nglString& rValue)
{
  mValue = nuiGetState(rValue);
  Changed();
  return true;
}

#endif // USE_NUIVALUETYPES

#endif // __nuiProperty_h__
