/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiRect_h__
#define __nuiRect_h__

#include "ngl.h"
#include "nuiFlags.h"
#include "nuiProperty.h"
#include "nuiColor.h"


typedef float nuiSize;
//typedef int nuiSize;

class nuiColor;

/// This class implements a 2D rectangle with many very usefull methods to manage them.
class nuiRect
#ifdef USE_NUIVALUETYPES  
  : public nuiValue
#endif
{
public:
  nuiRect()
  {
    Set(0,0,0,0,false);
  }

  nuiRect(nuiSize x1, nuiSize y1, nuiSize x2, nuiSize y2, bool SecondPairIsSize = true)
  {
    Set(x1,y1,x2,y2,SecondPairIsSize);
  }

  nuiRect(int x1, int y1, int x2, int y2, bool SecondPairIsSize = true)
  {
    Set(x1,y1,x2,y2,SecondPairIsSize);
  }

  nuiRect(const nuiRect& rRect)
  {
    mLeft = rRect.mLeft;
    mTop = rRect.mTop;
    mRight = rRect.mRight;
    mBottom = rRect.mBottom;
  }

  void Set(nuiSize x1 = 0.0f, nuiSize y1 = 0.0f, nuiSize x2 = 0.0f, nuiSize y2 = 0.0f, bool SecondPairIsSize = true)
  {
    if (SecondPairIsSize)
    {
      mLeft = x1;
      mTop = y1;
      mRight = x1 + x2;
      mBottom = y1 + y2;
    }
    else
    {
      mLeft = MIN(x1,x2);
      mTop = MIN(y1,y2);
      mRight = MAX(x1,x2);
      mBottom = MAX(y1,y2);
    }
#ifdef USE_NUIVALUETYPES  
    Changed();
#endif
  }

  void Set(int x1 = 0.0f, int y1 = 0.0f, int x2 = 0.0f, int y2 = 0.0f, bool SecondPairIsSize = true)
  {
    Set((nuiSize)x1, (nuiSize)y1, (nuiSize)x2, (nuiSize)y2,SecondPairIsSize);
  }

  void SetSize(int w, int h)
  {
    Set((nuiSize)mLeft, (nuiSize)mTop, (nuiSize)w, (nuiSize)h ,true);
  }

  void SetSize(nuiSize w, nuiSize h)
  {
    Set((nuiSize)mLeft, (nuiSize)mTop, (nuiSize)w, (nuiSize)h ,true);
  }

  inline nuiSize GetWidth() const
  {
    return mRight - mLeft;
  }

  inline nuiSize GetHeight() const
  {
    return mBottom - mTop;
  }

  inline void GetSize(nuiSize& rWidth, nuiSize& rHeight) const
  {
    rWidth  = GetWidth();
    rHeight = GetHeight();
  }

  inline void GetSize(int& rWidth, int& rHeight) const
  {
    rWidth  = (int)GetWidth();
    rHeight = (int)GetHeight();
  }

  inline nuiSize GetSurface() const
  {
    return GetWidth()*GetHeight();
  }

  inline nuiSize& Left()
  {
    return mLeft;
  }

  inline nuiSize& Right()
  {
    return mRight;
  }

  inline nuiSize& Top()
  {
    return mTop;
  }

  inline nuiSize& Bottom()
  {
    return mBottom;
  }

  inline void Move(nuiSize DeltaX, nuiSize DeltaY)
  {
    mLeft += DeltaX;
    mRight += DeltaX;
    mTop += DeltaY;
    mBottom += DeltaY;
#ifdef USE_NUIVALUETYPES  
    Changed();
#endif
  }

  inline void MoveTo(nuiSize X, nuiSize Y)
  {
    mRight += X - mLeft;
    mBottom += Y - mTop;
    mLeft = X;
    mTop = Y;
#ifdef USE_NUIVALUETYPES  
    Changed();
#endif
  }

  bool Intersect(const nuiRect& rRect1, const nuiRect& rRect2)
  {
    nuiSize dest_x, dest_y;
    nuiSize dest_w, dest_h;
    bool return_val;

    return_val = false;

    dest_x = MAX(rRect1.mLeft,   rRect2.mLeft  );
    dest_y = MAX(rRect1.mTop,    rRect2.mTop   );
    dest_w = MIN(rRect1.mRight,  rRect2.mRight ) - dest_x;
    dest_h = MIN(rRect1.mBottom, rRect2.mBottom) - dest_y;

    if (dest_w > 0 && dest_h > 0)
    {
      mLeft = dest_x;
      mTop = dest_y;
      mRight = mLeft + dest_w;
      mBottom = mTop + dest_h;
      return_val = true;
    }
    else
    {
      mLeft = 
      mRight = 
      mTop = 
      mBottom = 0;
    }

#ifdef USE_NUIVALUETYPES  
    Changed();
#endif
    return return_val;
  }

  void Union (const nuiRect& rRect1, const nuiRect& rRect2)
  {
    mLeft   = MIN(rRect1.mLeft,   rRect2.mLeft);
    mTop    = MIN(rRect1.mTop,    rRect2.mTop);
    mRight  = MAX(rRect1.mRight,  rRect2.mRight);
    mBottom = MAX(rRect1.mBottom, rRect2.mBottom);
#ifdef USE_NUIVALUETYPES  
    Changed();
#endif
  }

  nuiRect Size() const
  {
    nuiRect Rect;
    Rect.mRight  = GetWidth();
    Rect.mBottom = GetHeight();
    return Rect;
  }

  bool IsInside(nuiSize x, nuiSize y) const
  {
    return (x>=mLeft && x<=mRight && y>=mTop && y<=mBottom);
  }

  enum DrawMode
  {
    Lines,
    Fill
  };
/*                                                 
  virtual void Draw(DrawMode mode = Lines, nuiColor* pColor = NULL, nuiColor* pBorderColor = NULL) const
  {
    if (mode==Lines)
    {
      if (pColor)
        pColor->Apply();
      glBegin(GL_LINE_LOOP);
        glVertex2f(mLeft,mTop+1);
        glVertex2f(mRight-1,mTop+1);
        glVertex2f(mRight-1,mBottom);
        glVertex2f(mLeft,mBottom);
      glEnd();
    }
    else
    {
      if (pColor)
        pColor->Apply();

      glBegin(GL_QUADS);
        glVertex2f(mLeft,mTop);
        glVertex2f(mRight,mTop);
        glVertex2f(mRight,mBottom);
        glVertex2f(mLeft,mBottom);
      glEnd();

      if (pBorderColor) // draw border???
      {
        pBorderColor->Apply();
        glBegin(GL_LINE_LOOP);
          glVertex2f(mLeft,mTop+1);
          glVertex2f(mRight-1,mTop+1);
          glVertex2f(mRight-1,mBottom);
          glVertex2f(mLeft,mBottom);
        glEnd();
      }
    }
  }
*/

  void SetPosition(nuiPosition Position, const nuiRect& Inside)
  {
    nuiSize x,y,w,h;
    nuiSize rx,ry,rw,rh;
    rx = Inside.mLeft;
    ry = Inside.mTop;
    rw = Inside.GetWidth();
    rh = Inside.GetHeight();
    w=GetWidth();
    h=GetHeight();
    x = (rw/2) - (w/2);
    y = (rh/2) - (h/2);

    switch (Position)
    {
    case nuiCenter:
      mLeft = x;
      mRight = x+w;
      mTop = y;
      mBottom = y+h;
      break;
    case nuiFill:
      mLeft=0;
      mRight=rw;
      mTop=0;
      mBottom=rh;
      break;
    case nuiFillHorizontal:
      mLeft=0;
      mRight=rw;
      mTop=y;
      mBottom=y+h;
      break;
    case nuiFillVertical:
      mLeft=x;
      mRight=x+w;
      mTop=0;
      mBottom=rh;
      break;
    case nuiLeft:
      mLeft=0;
      mTop =y; 
      mRight=w;
      mBottom=y+h;
      break;
    case nuiRight:
      mLeft=rw-w;
      mTop =y; 
      mRight=rw;
      mBottom=y+h;
      break;
    case nuiTop:
      mLeft=x;
      mTop =0; 
      mRight=x+w;
      mBottom=h;
      break;
    case nuiBottom:
      mLeft=x;
      mTop =rh-h; 
      mRight=x+w;
      mBottom=rh;
      break;
    case nuiTopLeft:
      mLeft=0;
      mRight=w;
      mTop=0;
      mBottom=h;
      break;
    case nuiTopRight:
      mLeft=rw-w;
      mRight=rw;
      mTop=0;
      mBottom=h;
      break;
    case nuiBottomLeft:
      mLeft=0;
      mRight=w;
      mTop=rh-h;
      mBottom=rh;
      break;
    case nuiBottomRight:
      mLeft=rw-w;
      mRight=rw;
      mTop=rh-h;
      mBottom=rh;
      break;
    case nuiTile:
      mLeft = 0;
      mRight= w;
      mTop = 0;
      mBottom =h;
      break;
    }
#ifdef USE_NUIVALUETYPES  
    Changed();
#endif
  }

  // nuiVRect:
  virtual nglString GetValue()
  {
    nglString str;
    str.CFormat("{%f %f %f %f}",Left(), Top(), GetWidth(), GetHeight());
    return str;
  }

  virtual bool SetValue(const nglString& Value)
  {
    float x=0,y=0,w=0,h=0;
    int res;
    nglString str=Value;
    if (!str.GetLength() || str[0]!='{' || str[str.GetLength()-1]!='}')
      return false;
    str.DeleteLeft(1);
    str.DeleteRight(1);
    std::vector<nglString> vec;
    res = str.Tokenize(vec);
    if (vec.size() == 4)
    {
      x = vec[0].GetCFloat();
      y = vec[1].GetCFloat();
      w = vec[2].GetCFloat();
      h = vec[3].GetCFloat();
      Set(x,y,w,h);
#ifdef USE_NUIVALUETYPES  
      Changed();
#endif
      return true;
    }
    return false;
  }

  nuiRect& operator= (const nuiRect& Rect)
  {
    mLeft  = Rect.mLeft;
    mRight = Rect.mRight;
    mTop = Rect.mTop;
    mBottom = Rect.mBottom;
#ifdef USE_NUIVALUETYPES  
    Changed();
#endif
    return *this;
  }

  bool operator== (const nuiRect& Rect)
  {
    return (mLeft == Rect.mLeft && mRight == Rect.mRight && mTop == Rect.mTop && mBottom == Rect.mBottom);
  }

public:
  nuiSize mLeft,
      mRight,
      mTop,
      mBottom;
};

#endif // __nuiRect_h__
