/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiShape_h__
#define __nuiShape_h__

#include "ngl.h"
#include "nuiFlags.h"
#include "nuiColor.h"
#include "nuiSpline.h"
#include <list>

enum nuiShapeMode
{
  eStrokeShape,
  eFillShape,
  eStrokeAndFillShape,
  eDefault
};

class nuiContourElement
{
public:
  nuiContourElement();
  virtual ~nuiContourElement();

  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode) const; ///< This method saves the state in the given XML Node.
  //@}

  virtual bool Tesselate(std::list<nuiPoint>& rVertices, float Quality = 0.5f) = 0;

  nuiSimpleEventSource<nuiChanged> Changed; ///< This event is fired each time the ContourElement is changed.
};

// nuiContourLines:
class nuiContourLines : public nuiContourElement
{
public:
  nuiContourLines(const std::list<nuiPoint>& rVertices);
  nuiContourLines(const nuiPoint& rVertex);
  nuiContourLines(nuiXMLNode* pNode);
  virtual ~nuiContourLines();

  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode) const; ///< This method saves the state in the given XML Node.
  //@}

  virtual bool Tesselate(std::list<nuiPoint>& rVertices, float Quality = 0.5f);

  void AddLines(const std::list<nuiPoint>& rVertices);
  void LineTo(const nuiPoint& rVertex);

  uint GetPointsCount() const;
private:
  std::list<nuiPoint> mVertices;
};

// nuiContourRect:
class nuiContourRect : public nuiContourElement
{
public:
  nuiContourRect(const nuiRect& rRect, bool CCW = false);
  nuiContourRect(nuiXMLNode* pNode);
  virtual ~nuiContourRect();

  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode) const; ///< This method saves the state in the given XML Node.
  //@}

  virtual bool Tesselate(std::list<nuiPoint>& rVertices, float Quality = 0.5f);
private:
  nuiRect mRect;
  bool mCCW;
};

// nuiContourSpline:
class nuiContourSpline : public nuiContourElement
{
public:
  nuiContourSpline(const nuiSpline& rSpline);
  nuiContourSpline(nuiXMLNode* pNode);
  virtual ~nuiContourSpline();

  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode) const; ///< This method saves the state in the given XML Node.
  //@}

  virtual bool Tesselate(std::list<nuiPoint>& rVertices, float Quality = 0.5f);

private:
  nuiSpline mSpline;

  bool InvalidateCache(nuiEvent* pEvent);

  bool mCacheValid;
  float mLastQuality;
  nuiEventSink<nuiContourSpline> mContourSplineSink;
  std::list<nuiPoint> mTessCache;
};

class nuiContour
{
public:
  nuiContour();
  nuiContour(nuiXMLNode* pNode);
  virtual ~nuiContour();

  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode) const; ///< This method saves the state in the given XML Node.
  //@}

  void AddLines(const std::list<nuiPoint>& rVertices);
  void LineTo(const nuiPoint& rVertex);
  void AddRect(const nuiRect& rRect, bool CCW = false);

  void AddSpline(const nuiSpline& rSpline);

  void Close(bool closed); ///< This method permits to close the nuiContour. Only closed contours can be rendered as filled shapes. 
  bool IsClosed(); ///< This method returns true if the contour is closed (first point connected to the last point).

  virtual bool Tesselate(std::list<nuiPoint>& rVertices, float Quality = 0.5f);
  uint GetElementsCount() const;
  const std::list<nuiContourElement*>* GetElements() const;

  nuiSimpleEventSource<nuiChanged> Changed; ///< This event is fired each time the Contour is changed.
private:
  std::list<nuiContourElement*> mpElements;
  nuiContour(const nuiContour& rContour);
  bool ElementChanged(nuiEvent* pEvent);
  nuiEventSink<nuiContour> mEventSink;
  bool mClosed;
};


class nuiShape
{
public:
  nuiShape();
  nuiShape(nuiXMLNode* pNode);
  virtual ~nuiShape();

  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode) const; ///< This method saves the state in the given XML Node.
  //@}

  uint Acquire();
  uint Release();

  void AddContour(nuiContour* pContour);
  nuiContour* GetContour(uint Index);
  uint GetContourCount();

  virtual bool Tesselate(std::list<nuiPoint>& rVertices, float Quality = 0.5f);

  enum Winding 
  {
    eNone, ///< In this case the default winding rule is used.
    eOdd        = GLU_TESS_WINDING_ODD,
    eNonZero    = GLU_TESS_WINDING_NONZERO,
    ePositive   = GLU_TESS_WINDING_POSITIVE,
    eNegative   = GLU_TESS_WINDING_NEGATIVE,
    eAbsGEqTwo  = GLU_TESS_WINDING_ABS_GEQ_TWO
  };

  Winding GetWinding() const; ///< Set the Winding rule of this shape. The default winding rule is set to eNone (it will use the active winding rule of the draw context).
  void SetWinding(Winding Rule); ///< Get the Winding rule of this shape. 


  // Tesselation/Redering cache:
  class CacheElement
  {
  public:
    CacheElement(GLenum Operation, uint Count);
    virtual ~CacheElement();

    class Vertex
    {
    public:
      Vertex();
      virtual ~Vertex();

      float mCoord[3];
      float mColor[4];
      float mTexCoord[2];
      bool  mEdgeFlag;
    };

    Vertex* mpVertices;
    uint mCount;
    GLenum mOperation; // What operation should we do on these vertices.
  };
  std::list<class CacheElement*>* GetFillCache();
  std::list<class CacheElement*>* GetStrokeCache();
  void EmptyCaches();
  void AddCacheElement(nuiShapeMode ShapeMode,nuiShape::CacheElement* pElement);

  nuiSimpleEventSource<nuiChanged> Changed; ///< This event is fired each time the Shape is changed.

private:
  std::list<nuiContour*> mpContours;
  nuiShape(const nuiShape& rShape);
  uint mReferences;
  Winding mWinding;

  // Tesselation/Redering cache:
  std::list<CacheElement*> mpFillCache;
  std::list<CacheElement*> mpStrokeCache;

  bool ElementChanged(nuiEvent* pEvent);
  nuiEventSink<nuiShape> mEventSink;
};


#endif // __nuiShape_h__
