/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiSpline_h__
#define __nuiSpline_h__

#include "ngl.h"
#include "nglMath.h"
#include "nglVector.h"
#include "nglMatrix.h"
#include "nuiEvent.h"
#include "nuiRect.h"
#include "nuiPoint.h"
#include <list>
#include <vector>

class nuiSplineNode;
class nuiSplinePoint;

enum nuiSplineMode
{
  nuiSplineModeFree,
  nuiSplineModeCatmullRom,
  nuiSplineModeCardinal,
  nuiSplineModeTCB,
  nuiSplineModeBezier
};

class nuiSplineNode
{
public:
  /** @name Life */
  //@{
  nuiSplineNode(float x, float y, float z = 0);
  nuiSplineNode(const nuiSplineNode& rNode);
  nuiSplineNode(nuiXMLNode* pNode); ///< Load from an XML Node
  virtual ~nuiSplineNode();
  //@}

  /** @name Serialisation */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pNode, nuiSplineMode Mode = nuiSplineModeFree) const; ///< This method saves the state in the given XML Node.
  //@}

  void SetPosition(const nglVectorf& rVector); ///< Change the position of the control point.
  const nglVectorf& GetPosition() const; ///< Get the position of the control point.

  /** @name TCB parameters */
  //@{
  void SetTension(float Tension);        ///< Set the tension parameter of this node.
  float GetTension();                    ///< Return the bias parameter of this node.
  void SetBias(float Bias);              ///< Set the bias parameter of this node.
  float GetBias();                       ///< Return the bias parameter of this node.
  void SetContinuity(float Continuity);  ///< Set the continuity parameter of this node.
  float GetContinuity();                 ///< Return the continuity parameter of this node.
  //@}

  /** @name Low level tangent access */
  //@{
  void SetTangent(const nglVectorf& rTangent);   ///< Set the tangent parameter of this node (will update both the incomming and outgoing tangents).
  void SetIncomingTangent(const nglVectorf& rTangent);   ///< Set the incomming tangent parameter of this node.
  void SetOutgoingTangent(const nglVectorf& rTangent);   ///< Set the outgoing tangent parameter of this node.
  nglVectorf GetTangent();                       ///< Return the tangent parameter of this node (equivalent to GetIncommingTangent).
  nglVectorf GetIncomingTangent();               ///< Return the incoming tangent parameter of this node.
  nglVectorf GetOutgoingTangent();               ///< Return the outgoing tangent parameter of this node.
  void RotateTangent(float angle, float x, float y, float z); ///< Rotate both tangents. 
  void RotateIncomingTangent(float angle, float x, float y, float z); ///< Rotate the incoming tangents. 
  void RotateOutgoingTangent(float angle, float x, float y, float z); ///< Rotate the outgoing tangents. 
  void ScaleTangent(float x, float y, float z); ///< Scale both tangents. 
  void ScaleIncomingTangent(float x, float y, float z); ///< Scale the incoming tangents. 
  void ScaleOutgoingTangent(float x, float y, float z); ///< Scale the outgoing tangents. 
  //@}

  /** @name Copy Operator */
  //@{
  nuiSplineNode& operator = (const nuiSplineNode& rNode);  ///< Copy operator.
  const nglVectorf& operator = (const nglVectorf& rVector); ///< Copy operator.
  //@}

  /** @name Outgoing events */
  //@{
  nuiSimpleEventSource<nuiChanged> Changed; ///< This event is fired each time the Spline is changed.
  //@}

  /** @name Meta data management: */
  //@{
  void SetMetaData(const std::vector<float>& Data);
  void GetMetaData(std::vector<float>& Data) const;
  //@}

  /** @name Meta data helper methods: */
  //@{
  void SetColor(const nuiColor& Color)
  {
    if (mMetaData.size() < NUI_POINT_COLOR_A+1)
      mMetaData.resize(NUI_POINT_COLOR_A+1);
    mMetaData[NUI_POINT_COLOR_R] = Color.GetRed();
    mMetaData[NUI_POINT_COLOR_G] = Color.GetGreen();
    mMetaData[NUI_POINT_COLOR_B] = Color.GetBlue();
    mMetaData[NUI_POINT_COLOR_A] = Color.GetAlpha();
  }

  nuiColor GetColor() const
  {
    if (mMetaData.size() < NUI_POINT_COLOR_A+1)
    {
      // If there was no color yet fill with black:
      mMetaData.resize(NUI_POINT_COLOR_A+1);
      mMetaData[NUI_POINT_COLOR_A] = 1;
    }
    return nuiColor(mMetaData[NUI_POINT_COLOR_R],
                    mMetaData[NUI_POINT_COLOR_G],
                    mMetaData[NUI_POINT_COLOR_B],
                    mMetaData[NUI_POINT_COLOR_A]);
  }

  void SetTexCoord(const nglVector2f& TexCoord)
  {
    if (mMetaData.size() < NUI_POINT_TEXCOORD_Y+1)
      mMetaData.resize(NUI_POINT_TEXCOORD_Y+1);
    mMetaData[NUI_POINT_TEXCOORD_X] = TexCoord.Elt[0];
    mMetaData[NUI_POINT_TEXCOORD_Y] = TexCoord.Elt[1];
  }

  nglVector2f GetTexCoord() const
  {
    if (mMetaData.size() < NUI_POINT_TEXCOORD_Y+1)
      mMetaData.resize(NUI_POINT_TEXCOORD_Y+1);
    return nglVector2f(mMetaData[NUI_POINT_TEXCOORD_X], mMetaData[NUI_POINT_TEXCOORD_Y]);
  }
  //@}

protected:
  nglVectorf mPosition;
  nglVectorf mTangentIn;
  nglVectorf mTangentOut;
  float mTension;
  float mContinuity;
  float mBias;

  mutable std::vector<float> mMetaData;
};


class nuiSpline
{
public:
  nuiSpline();
  nuiSpline( const nuiSpline& rSpline ); ///< Copy constructor.
  nuiSpline(nuiXMLNode* pNode); ///< Load from an XML Node
  virtual ~nuiSpline();

  
  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pNode) const; ///< This method saves the state in the given XML Node.
  //@}

  void InsertNode(const nuiSplineNode& rNode, uint32 index = 0); ///< Insert a Node @param rNode at @param index in the Spline.
  void InsertNodes(std::list<nuiSplineNode> rNodes, uint32 index = 0); ///< Insert the Nodes in the list @param rNodes at @param index in the Spline.

  void AddNode(const nuiSplineNode& rNode); ///< Add a Node @param rNode at the end of the Spline.
  void AddNodes(std::list<nuiSplineNode> rNodes); ///< Add the Nodes in the list @param rNodes at the end of the Spline.

  void DeleteNode(uint32 index); ///< Delete the Node at the given @param index.
  void DeleteNodes(uint32 index,uint32 count); ///< Delete @param count Nodes starting at Node @param index.
  void DeleteNodes(std::list<uint32> indices); ///< Delete the Node whose index is in the list @param indices.
  void DeleteAll(); ///< Delete all Nodes.

  nuiSplineNode* GetNode(uint32 index); ///< Return the Node at index @param index in the Spline.
  nuiSplineNode* GetNode(nglVectorf vector, float maxdist = 5 ); ///< Return a reference to the closest Node to the given @param vector.
  uint32 GetNodes(std::list<nuiSplineNode>&rNodes, uint32 index = 0, int32 count = -1); ///< Copy @param count Nodes from the index @index in the list to the list @param rNodes. If count == -1 then it will copy all the Nodes. Return the number of Nodes actually copied.

  virtual nuiSplinePoint Evaluate(float Value); ///< Evaluate the spline at the given Value. The evaluation goes from trunc(Value) to trunc(Value)+1.
  virtual void Evaluate(std::list<nuiSplinePoint>& rPoints, uint steps); ///< Evaluate the spline as steps point and put them in the given list.
  virtual void PrecalcTangents(); ///< Prepare the spline for evaluation. Call this method at least once before calling Evaludate(float Value). The other Evaluate method calls it directly.

  uint32 GetSize(); ///< Return the number of Nodes actually contained in the Spline.

  nuiRect GetRect(); ///< Return the bounding rect around the Nodes.

  void Translate(const nglVectorf& rTranslation); ///< Translate the Nodes along the given Node.
  void Rotate(float angle, float x, float y, float z);  ///< Rotate the Nodes (see glRotate for the significance of the parameters).
  void Scale(float x, float y, float z);  ///< Scale the Nodes along each axis given the respecting values.
  void Multiply(const nglMatrixf& matrix); ///< Multiply every Node of the Spline by the given matrix.

  bool IsClosed(); ///< Returns true if the Spline is to be closed (end Node == start Node).
  void SetClosed(bool closed = true); ///< Enables to close the shape (end Node == start Node).

  nuiSimpleEventSource<nuiChanged> Changed; ///< This event is fired each time the Spline is changed.

  void SetCardinalMode(float tightness); ///< Set the spline in Cardinal mode with the given tightness.
  void SetCatmullRomMode(); ///< Set the spline in Catmull-Rom mode (equivalent to SetCardinalMode(0.5) ).
  void SetTCBMode(); ///< Set the spline in Tension-Continuity-Bias mode. 
  void SetFreeMode(); ///< Set the spline in free mode: the tangents won't be recalculated automatically.
  void SetBezierMode(); ///< Set the spline in Bezier mode: the tangents won't be recalculated automatically and they are in fact the second and third control points in a cubic bezier spline.
  nuiSplineMode GetMode();

  nuiSpline& operator=(const nuiSpline& rSpline);
protected: 
  bool mClosed;
  std::list <nuiSplineNode> mNodes;
  nuiSplineMode mMode;
  float mCardinalTightness;

  bool mChanged;
  nuiSimpleEventSource<nuiChanged> InternalChanged; ///< This event is fired each time the Spline is changed internally.
  bool ElementChanged(nuiEvent* pEvent);
  nuiEventSink<nuiSpline> mEventSink;

  void CalculateCardinalTangents(float tightness); ///< Calculate the Tangents with the Cardinal algorithm.
  void CalculateCatmullRomTangents(); ///< Calculate the Tangents with the Cardinal algorithm (cardinal with a tightness of 0.5).
  void CalculateTCBTangents(); ///< Calculate the Tangents with the TCB algorithm.
private:
};


class nuiSplinePoint: public nuiPoint
{
public:
  nuiSplinePoint() : nuiPoint()
  { 
  }

  /// Initialize with values
  nuiSplinePoint(float X, float Y, float Z, float W = 1.f) : nuiPoint(X,Y,Z,W)
  {
  }

  /// Initialize with a two dimensional vector. Third element is zero, fourth is one.
  nuiSplinePoint(const nglVector2f& rVector) : nuiPoint(rVector)
  { 
  }

  /// Initialize with a two dimensional vector. Fourth element is one.
  nuiSplinePoint(const nglVector3f& rVector)  : nuiPoint(rVector)
  { 
  }

  /// Initialize with a four dimensional vector.
  nuiSplinePoint(const nglVectorf& rVector)  : nuiPoint(rVector)
  { 
  }

  /// Initialize with a four dimensional vector.
  nuiSplinePoint(const nuiSplineNode& rNode)  : nuiPoint(rNode.GetPosition())
  { 
    rNode.GetMetaData(mMetaData);
  }

  /// Copy constructor
  nuiSplinePoint(const nuiSplinePoint& rPoint)  : nuiPoint(rPoint)
  {
    rPoint.GetMetaData(mMetaData);
  }

  /// Destructor
  virtual ~nuiSplinePoint() ///< Destructor.
  {
  }
};

#endif // __nuiSpline_h__
