/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiWidget_h__
#define __nuiWidget_h__

#include "nuiObject.h"
#include "nuiRect.h"
#include "nglWindow.h"
#include "nuiMouseEvent.h"

class nuiDragAndDrop;

class nuiContainer;
class nuiDrawContext;
class nuiWidget;

typedef nuiWidget* nuiWidgetPtr;
typedef std::list<nuiWidgetPtr> nuiWidgetList;

typedef nuiContainer* nuiContainerPtr;
typedef std::list<nuiContainerPtr> nuiContainerList;


class nuiWidget : public nuiObject
{
public:
  /** @name Life */
  //@{
  nuiWidget(nuiContainerPtr pParent); ///< Create an nuiObject and give him his parent.
  nuiWidget(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create an nuiObject from an XML description.
  virtual ~nuiWidget(); 
  //@}

  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive = false);
  //@}


  /** @name Object relation management */
  //@{
  virtual nuiContainerPtr GetParent(); ///< Return the parent of this object
  virtual nuiContainerPtr GetRoot(); ///< Return the top parent of this object. 
  virtual bool SetParent(nuiContainerPtr pParent); ///< Set the parent of this object. 
  virtual bool IsParentActive(); ///< Return true if this object main parent is an active window.
  virtual bool Trash(); ///< This method will try to destroy the nuiObject as soon as possible. Returns true if the destruction is ongoing, false if there was a problem. Use this method instead of the delete C++ keyword all the time!!!!
  virtual bool HasInheritedProperty (const nglString& rName); ///< This method returns true if a widget or one of its parents has the given property.
  virtual nglString GetInheritedProperty (const nglString& rName); ///< Returns the value of the given property. This method looks for the property in the object and in its parents recursively.
  virtual nuiWidgetPtr Find (const nglString& rName);  ///< Finds a node given its full path relative to the current node. Eg. Find("background/color/red").
  //@}

  /** @name Coordinates translation */
  //@{
  virtual void LocalToGlobal(int& x, int& y); ///< This method recursively calculate the coordinate ot the given point if the system of the top parent. (x,y) will be translated from the client coordinates to the main window coordinates.
  virtual void LocalToGlobal(nuiSize& x, nuiSize& y); ///< This method recursively calculate the coordinate ot the given point if the system of the top parent. (x,y) will be translated from the client coordinates to the main window coordinates.
  virtual void LocalToGlobal(nuiRect& rRect);
  virtual void GlobalToLocal(int& x, int& y); ///< This method recursively calculate the coordinate ot the given point if the system of the top parent. (x,y) will be interpretted as main window coordinate and translated to client coordinates.
  virtual void GlobalToLocal(nuiSize& x, nuiSize& y); ///< This method recursively calculate the coordinate ot the given point if the system of the top parent. (x,y) will be interpretted as main window coordinate and translated to client coordinates.
  virtual void GlobalToLocal(nuiRect& rRect);
  virtual void LocalToLocal(nuiWidgetPtr pWidget, int& x, int& y); ///< This method translates (x,y) from the Local coordinate system of this object to the local coordinate system of pWidget. They must be both owned by the same top level object.
  virtual void LocalToLocal(nuiWidgetPtr pWidget, nuiSize& x, nuiSize& y); ///< This method translates (x,y) from the Local coordinate system of this object to the local coordinate system of pWidget. They must be both owned by the same top level object.
  virtual void LocalToLocal(nuiWidgetPtr pWidget, nuiRect& rRect); ///< This method translates rRect from the Local coordinate system of this object to the local coordinate system of pWidget. They must be both owned by the same top level object.
  //@}

  /** @name Object size management */
  //@{
  virtual nuiRect CalcIdealSize(); ///< This method asks the object to recalculate its layout freely. On return mIdealRect is modified to contain the bounding rectangle needed by the widget and returned to the caller. You will probably have to override this method if you create new widgets, how ever you are never supposed to call it directly and use GetIdealRect instead.
  virtual bool SetRect(const nuiRect& rRect); ///< This method asks the object to recalculate its layout and to force using the given bounding rectangle. You will probably have to overriden this method if inherited widgets, you should then call the original nuiWidget::SetRect as soon as possible when entering the new method. Failure to do so will be prosecuted. 

  virtual nuiRect GetIdealRect(); ///< Return the ideal area used by this Object. If the layout of this object has changed CalIdealRect will be called and mIdealRect will contain the ideal rectangle. If the user specified a user size then mIdealRect will be overwritten with mUserRect. 
  virtual nuiRect GetRect(); ///< Return the current area used by this Object.

  void SetUserRect(const nuiRect& rRect); ///< Ask the widget to use the specified rect as its ideal size and position. The position and size may not be granted by the widgets parent. This method is a combination of SetUserPos and SetUserSize.
  void SetUserPos(nuiSize X, nuiSize Y); ///< Ask the widget to use the specified position as its ideal position. The position and size may not be granted by the widgets parent.
  void SetUserSize(nuiSize Width, nuiSize Height); ///< Ask the widget to use the specified size as its ideal size. The position and size may not be granted by the widgets parent.
  void UnsetUserRect(); ///< Ask the widget to stop using the user provided rectangle and restart calculating its size on its own.
  void UnsetUserPos(); ///< Ask the widget to stop using the user provided position and restart calculating its position on its own.
  void UnsetUserSize(); ///< Ask the widget to stop using the user provided size and restart calculating its size on its own.
  bool HasUserRect(); ///< Returns true if the widget is using a user provided rectangle to calculate its size and position.
  bool HasUserPos(); ///< Returns true if the widget is using a user provided point to calculate its position.
  bool HasUserSize(); ///< Returns true if the widget is using a user provided size to calculate its size.
  void ForceIdealRect(bool Force); ///< Always. Misuse of this flag can lead to very strange visual bugs so beware!

  virtual void ValidateLayout(); ///< This method reset the mLayoutChanged flag to false. It must be called by any object in the SetRect() method.
  virtual bool HasLayoutChanged(); ///< This method returns true if the layout of the object has changed since the last layout/redraw loop.

  virtual bool IsInside(nuiSize X, nuiSize Y); ///< Return true if the point (X,Y) (in the coordinates of the root object) is inside the object. This method call IsInsideLocal internally so you may not need to redefine it.
  virtual bool IsInsideLocal(nuiSize X, nuiSize Y); ///< Return true if the point (X,Y) (in the coordinates of the parent) is inside the object.
  //@}

  /** @name Rendering */
  //@{
  virtual bool Draw(nuiDrawContext* pContext); ///< This method asks the object to draw itself. It returns false in case of error. You must call Validate() once in this method if you decide to override it. You must not draw the widget if it is not visible (check the result of IsVisible() before drawing anything but after having called Validate()).
  virtual void Invalidate(bool LayoutChanged = false, bool TellChildren = false); ///< Ask for a redraw of the object. Only the nuiMainWindow class should redefine this method.
  virtual void Validate(); ///< Reset the invalidate flag as the object doesn't need Redrawing anymore. ATTENTION: you must call this method once in the draw method or any Invalidate() call will be ignored (and no redraw possible)...
  virtual float GetAlpha(bool MixWithParent = true); ///< Returns the current alpha transparency value of this object, mixed with the one of its parents if MixWithParent==true.
  virtual void SetAlpha(float Alpha); ///< The the alpha transparency value of this object. 
  virtual nuiDrawContext* GetDrawContext(); ///< Retreive the draw context from the root object (the main window).
  //@}

  /** @name Incomming keyboard events */
  //@{
  virtual bool IsKeyDown (nglKeyCode Key);
  virtual bool KeyDown (nglKeyCode Key, nglChar Char);
  virtual bool KeyUp   (nglKeyCode Key, nglChar Char);
  //@}

  /** @name Incomming mouse events */
  //@{
  // These three methods receive the mouse coordinates in this object referential
  virtual bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseUnclicked(nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseMoved    (nuiSize X, nuiSize Y);
  virtual bool MouseGrabbed();
  virtual bool MouseUngrabbed();
  //@}

  /** @name Other Incomming events */
  //@{
  virtual void OnSetFocus(nuiWidgetPtr pWidget); ///< Advise the objet of a change of focus object. pWidget can be null.
  //@}

  /** @name Misc. & incomming events management */
  //@{
  virtual bool Grab();  ///< Request the mouse grab
  virtual bool Ungrab(); ///< Release the mouse grab
  virtual bool HasFocus(); ///< True if the object has the focus.
  virtual bool Focus(); ///< Ask the system to put the keyboard focus on this widget so that it can receive keyboard events. 
  virtual bool UnFocus(); ///< Ask the system to remove the keyboard focus from this widget so that it can not receive keyboard events anymore. 
  virtual bool SetMouseCursor(nglCursorType Cursor); ///< Set the mouse cursor of the object (when the mouse is hovering on the object). The default cursor is eCursorArrow.
  virtual nglCursorType GetMouseCursor(); ///< Get the mouse cursor of the object (when the mouse is hovering on the object). The default cursor is eCursorArrow.
  //@}

  /** @name State & flags management */
  //@{
  virtual bool GetHover(); ///< Return true if the widget is in hover state (most of the time it is when the mouse hovers over the widget)
  virtual void SetHover(bool Hover); ///< Set the hover state of the object and its children. Most application don't need to use this method directly. Will generate an invalidation of the rendering.
  virtual nuiState GetState(bool combined = true); ///< Return the current state of the object (Normal, Selected or Disabled). By default the returned state represents the final state of the object combined with the states of its hierarchy. If it is not combined then the returned value is le local state of the node.
  virtual void SetState(nuiState State); ///< Change the current state of the object and all its children. Will generate an invalidation of the rendering.
  virtual void LockState(); ///< Lock the state of the object. The state can't be changed until it is unlocked, also, the object will not combine its state with its parent's anymore.
  virtual void UnlockState(); ///< Unlock the object's state. See LockState() for for info. 
  virtual void SetVisible(bool Visible); ///< Show or hide the widget
  virtual bool IsVisible(bool combined = true); ///< Return true if the object is visible. By default the returned state will be the state of the object combined with the states of its hierarchy recursively. If one of its parents is invisible the widget becomes invisible. If combine==false the returned value will be the local visible state of the object.
  virtual void SetSelectionExclusive(bool Exclusive); ///< When an object is set to exclusive selection it will deselect all its slibblings before selecting itself (for easy radio buttons and the like).
  virtual bool IsSelectionExclusive(); ///< Return true if the object is part of a selection group.
  //@}


  /** @name Generic tooltips */
  //@{
  virtual bool SetToolTip(const nglString& rToolTip); ///< Change the text of the tooltip.
  virtual nglString GetToolTip(); ///< Retreive the text of the tooltip.
  virtual bool ActivateToolTip(nuiWidgetPtr pWidget); ///< The user doesn't need to use this method directly.
  virtual bool ReleaseToolTip(nuiWidgetPtr pWidget); ///< Tell the system to stop displaying this object's tooltip.
  //@}

  /** @name Drag and drop */
  //@{
//  void CancelDrag(nuiDragObject* pDragObject); ///< A widget calls this method to stop an ongoing drag and drop operation with the designated object. 
  virtual bool OnCanDrop(nuiDragAndDrop* pDragObject,nuiSize X,nuiSize Y); ///< This method is called by the window manager whenever the mouse is dragging an object over this widget. This method should test the object for supported drag and drop types and return true if one of them is supported, false otherwise. 
  virtual void OnCancelDrag(nuiDragAndDrop* pDragObject); ///< This method is called on the drag and drop source widget by the window manager whenever the d&d operation was canceled (for exemple by releasing the mouse button on a widget that doens't support the dragged object type). 
  virtual void OnDragged(nuiDragAndDrop* pDragObject); ///< This method is called on the drag and drop source widget by the window manager whenever the drag & drop operation was accepted by the user (by releasing the mouse button on a widget that support the dragged object type). This is the last time the source widget is allowed to place data in the drag and dropped object. 
  virtual void OnDropped(nuiDragAndDrop* pDragObject,nuiSize X,nuiSize Y, nglMouseFlags Button); ///< This method is called by the window manager whenever the mouse is dragging an object over this widget. This method should test the object for supported drag and drop types and return true if one of them is supported, false otherwise. The mouse flag argument contains the mouse event that initiated the drop operation. X & Y are the coordinates of the mouse inside the destination widget.
  //@}

  /** @name Basic outgoing events */
  //@{
  nuiSimpleEventSource <nuiTrashed> Trashed; ///< Send an event when the object is destroyed
  nuiSimpleEventSource <nuiDestroyed> Destroyed; ///< Send an event when the object is destroyed
  nuiSimpleEventSource <nuiWidgetParentChanged> ParentChanged; ///< Send an event when the object is reparented

  nuiSimpleEventSource <nuiWidgetSelected    > Selected; ///< Send an event when the object is deselected
  nuiSimpleEventSource <nuiWidgetDeselected  > Deselected; ///< Send an event when the object is selected
  nuiSimpleEventSource <nuiWidgetDisabled    > Disabled; ///< Send an event when the object is disabled
  nuiSimpleEventSource <nuiWidgetStateChanged> StateChanged; ///< Send an event when the object change its state (Normal, Selected or Disabled)

  nuiSimpleEventSource <nuiWidgetShown> Shown; ///< Send an event when the object is shown
  nuiSimpleEventSource <nuiWidgetHiden> Hiden; ///< Send an event when the object is hiden
  nuiSimpleEventSource <nuiWidgetVisibilityChanged> VisibilityChanged; ///< Send an event when the object is either hiden or shown.

  nuiSimpleEventSource <nuiWidgetHoverOn     > HoverOn; ///< Send an event when the mouse is hovering on the object.
  nuiSimpleEventSource <nuiWidgetHoverOff    > HoverOff; ///< Send an event when the mouse stops hovering on the object.
  nuiSimpleEventSource <nuiWidgetHoverChanged> HoverChanged; ///< Send an event when the hovering state changes.

  nuiMouseClicked Clicked; ///< Send an event when the widget is clicked. This event is fired after the MouseClicked method is called on the widget.
  nuiMouseUnclicked Unclicked; ///< Send an event when the widget is clicked. This event is fired after the MouseUnclicked method is called on the widget.
  //@}

  /** @name Private event management system (do not override unless you know what you're doing!!!) */
  //@{
  /// Beware: these three methods receive the mouse coordinates in the repair of the root object!
  virtual bool DispatchMouseClick  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool DispatchMouseUnclick(nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool DispatchMouseMove   (nuiSize X, nuiSize Y);
  //@}

protected:
  void InitProperties();

  nuiContainerPtr mpParent;

  nuiFloat mAlpha; ///< Indicates the transparency level of the object. Optional. 
  nuiRect mRect; ///< The bounding box of the nuiObject (in coordinates of its parent).
  nuiRect mIdealRect; ///< The ideal bounding box of the nuiObject (in coordinates of its parent) position should be at the origin.
  nuiRect mUserRect; ///< The bounding box of the nuiObject if set by the user (in coordinates of its parent).
  nuiBool mHasFocus; ///< True if the widget has the focus.
  nuiBool mLayoutChanged;
  nuiBool mNeedRender;
  nuiBool mVisible;
  nuiBool mHasUserPos;
  nuiBool mHasUserSize;
  nuiBool mForceIdealSize;
  nuiBool mSelectionExclusive;

  nuiVState mState;
  nuiBool mStateLocked;
  nuiBool mHover;

  nglCursorType mObjCursor;

  friend class nuiContainer;
};


#endif // __nuiWidget_h__
