/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiWindow_h__
#define __nuiWindow_h__

#include "ngl.h"
#include "nuiContainer.h"
#include "nuiImage.h"
#include "nuiEvent.h"
#include "nuiMouseEvent.h"

class nuiButton;
class nuiMainWindow;

/// A nui window is an overlapped popup window. It can only be owned by a MainWindow.
class nuiWindow : public nuiContainer
{
public:
  // nuiWindow flags:
  static const nuiWindowFlags NoFlag;
  static const nuiWindowFlags StaticChildren; ///< The window's children are static and do not occupy all the window space but the size and position they are given at creation.
  static const nuiWindowFlags NoMove;         ///< The window cannot be moved by the user, even if it have a caption bar.
  static const nuiWindowFlags NoResize;       ///< The window can't be resized by the user.
  static const nuiWindowFlags Raw;            ///< The window is raw and will not draw it self. Its children have to do this job. This flag implies NoCaption.
  static const nuiWindowFlags NoCaption;      ///< The window have no caption bar. It implies also the NoClose flag.
  static const nuiWindowFlags NoClose;        ///< The window have no to close button.
  static const nuiWindowFlags Modal;          ///< The window if modal: no other window created before it can receive any mouse of keyboard event until it is destroyed.

  // Construction/Destruction
  nuiWindow(nuiMainWindow* pParent, const nuiRect& rRect, nuiWindowFlags Flags=NoFlag, nglString Title=nglString(_T("nuiWindow")));
  nuiWindow(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  virtual ~nuiWindow();

  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive);

  // Parenting:
  bool SetParent(nuiMainWindow* pParent);
  virtual bool IsParentActive(); ///< Return true if this object is the active object.

  // Rendering & theme ops:
  virtual bool Draw(nuiDrawContext* pContext);
  virtual nuiRect CalcIdealSize();
  virtual bool SetRect(const nuiRect& rRect);
  virtual bool IsInside(nuiSize X, nuiSize Y);

  // Background methods:
  virtual void SetBackground(nuiImage* pImage); ///< set image bg color. 
  virtual void SetBackground(nuiColor& rColor); ///< Set unique bg color.
  virtual void SetBackground(nuiColor rColor[4]); ///< Set gradiant bg color.
  virtual void SetUseBitmapBg(bool bg);
  virtual bool GetUseBitmapBg();
  virtual void SetUseSolidBg(bool bg);
  virtual bool GetUseSolidBg();
  virtual nuiImage* GetBackgroundImage(); ///< set image bg color. 
  virtual nuiColor* GetBackgroundColor(); ///< Set gradiant bg color.

  // Events callbacks:
  virtual void OnActivation();    ///< Called when the window gains the keyboard focus.
  virtual void OnDesactivation(); ///< Called when the window looses the keyboard focus.

  virtual bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseUnclicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseMoved  (nuiSize X, nuiSize Y);

  // Flags:
  virtual void SetFlags(nuiWindowFlags Flags);
  virtual nuiWindowFlags GetFlags();
  virtual bool IsModal(); ///< Returns true if the window is modal. Every window is modeless by default.
  virtual bool SetModal(bool modal = false); ///< Changes the behaviour of the window (modal or modeless). Returns true if the modal flag change succeded. Every window is modeless by default.
  virtual void SetTitle(const nglString& rTitle); ///< Change the title of the window.
  virtual nglString GetTitle(); ///< Get the title of the window.

  // nui Events:
  nuiMouseClicked   ClickedMouse;
  nuiMouseUnclicked UnclickedMouse;
  nuiMouseMoved     MovedMouse;
  nuiSimpleEventSource<nuiButtonPressed> CloseButtonPressed;

protected:
  friend class nuiMainWindow;
  virtual bool DispatchMouseClick (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool DispatchMouseUnclick(nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool DispatchMouseMove   (nuiSize X, nuiSize Y);

  void InitProperties();

  bool mBitmapBackGround;
  bool mSolidBackGround;
  nuiColor mBgColor[4]; // One color for each corner
  nuiImage* mpBackground; // Background image

  nuiSize mClickX, mClickY; ///< Mouse coordinates captured from last click.
  nuiRect mClickRect;   ///< Window rect captured from last click.
  enum {
    eNoMove,
    ePreMove,
    eMove
  } mMoving;
  bool mModal; ///< True if the window is modal. Every window is modeless by default.
  bool mStaticChildren; ///< This flag is true if the children keep their original size and positions.
  bool mNoResize;
  bool mNoBorder;
  bool mNoCaption;
  bool mNoClose;
  bool mNoMove;
  bool mRawWindow;
  nglString mTitle;
  nuiButton* mpCloseButton;
  nuiPosition mClickPos; ///< Indicate where the last click was (Center is inside the client area, Fill is the caption bar, the other values are the border if the window is resizable).
  nuiSize mMinimumHeight;
  nuiSize mMinimumWidth;
private:
  bool HandleCloseButtonPressed(nuiEvent* pEvent);

  bool SetParent(nuiContainerPtr pParent) { return false; }


  nuiEventSink<nuiWindow> mNuiWindowSink;
};

//typedef nuiSmartPtr<nuiWindow> nuiWindowPtr;
typedef nuiWindow* nuiWindowPtr;
typedef std::list<nuiWindowPtr> nuiWindowList;

#endif // __nuiWindow_h__
