/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiXML_h__
#define __nuiXML_h__

#include "ngl.h"
#include "nglString.h"
#include "nglIFile.h"
#include "nglOFile.h"
#include "nuiApplication.h"
#include "nuiContainer.h"

#include <list>
#include <map>

class xmlLexer;

/// This class implements a very basic (but useful) XML node based on nglString. You will probaly use it in conjonction with the nuiXML class.
class nuiXMLNode
{
public:
  nuiXMLNode(const nglString& rName, nuiXMLNode* pParent=NULL); ///< Create a node with the given name and attach it to its parent if it is not NULL.
  virtual ~nuiXMLNode();

  // Node management:
  nglString GetName(); ///< Return the name of the node.
  void      SetName(const nglString& rName); ///< Set the name of the node.

  nglString GetValue(); ///< Return the text value of the node or an empty string if the node is not a text node (its name != ##text).
  void      SetValue(const nglString& rValue); ///< Transforms the node into a text node if it is not allready one and sets its text.

  // Node attributes management:
  
  void      SetAttribute(const nglString& rName, const nglString& rValue); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, const nglChar* pValue); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, uint value); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, int value); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, bool value); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, float value); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, double value); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, nuiPosition value); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, nuiState value); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, nuiDirection value); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.
  void      SetAttribute(const nglString& rName, nuiOrientation value); ///< Set the value of an attribute given its name. If the attribute doesn't exits on this object it will be added and its value updated.

  bool      DelAttribute(const nglString& rName); ///< Remove an attribute from this node. If the attribute doesn't exists on the node nothing happens.

  bool      HasAttribute(const nglString& rName); ///< Return true if the given attribute is present on the node.
  
  nglString GetAttribute(const nglString& rName); ///< Return the value of the given attribute. If the attribute doesn't exists on this object the returned string will be empty.
  uint      GetAttributeCount(); ///< Return the number of attributes presents on this object.
  nglString GetAttributeName (uint index); ///< Return the name of the attribute with the given index.
  nglString GetAttributeValue(uint index); ///< Return the value of the attribute with the given index.

  // Node tree management:
  const std::list<nuiXMLNode*>& GetChildren(); ///< Return a copy of the list of children of this node. You must not change any child directly from this list. Removing or adding a child from the returned list will not remove if from the node, you must use the other node methods to do that. You must not change a node directly either.
  bool AddChild(nuiXMLNode* pNode); ///< Add the given node to the list of children. If the child is allready there nothing happens. Return true on success.
  bool DelChild(nuiXMLNode* pNode); ///< Remove the given node from the list of children.
  uint GetChildrenCount(); ///< Return the number of children of this node.
  nuiXMLNode* GetChild(uint index); ///< Return the child with the given index on this node's children list.
  nuiXMLNode* GetChild(const nglString& rName); ///< Return the child with the given name on this node's children list.

  nuiXMLNode* Find(const nglString& rName); ///< Finds a node given its full path relative to the current node. Eg. Find("background/color/red").

  // Debug:
  nglString Dump(uint level); ///< Output the tree in the console (set the tab indent to \param level)..

protected:
  nuiXMLNode(xmlLexer* pLexer, nuiXMLNode* pParent);
  virtual bool ParseXMLNode(xmlLexer* pLexer, nglString& str);
  virtual bool ParseXMLCommand(xmlLexer* pLexer, nglString& str);

  nglString mName;
  nglString mValue;
  std::map<nglString,nglString, nglString::LessFunctor> mAttributes;
  std::list<nuiXMLNode*> mpChildren;
private:
  nuiXMLNode(const nuiXMLNode& rOriginal);
};

/// This class implements a very basic (but useful) XML parser/saver based on nglString. Is the basis of the xml widget tree loading/saving scheme.
class nuiXML : public nuiXMLNode
{
public:
  nuiXML(const nglString& rName);
  virtual ~nuiXML();

  virtual bool Load (const nglIStream& rStream); ///< Load an existing xml file from a stream.
  virtual bool Save (const nglOStream& rStream); ///< Save the current tree to a stream.
 
  virtual nglString Dump(); ///< Return the contents of the object as a string.
  virtual void SetDTD(nglString name, nglString file); ///< Set the DTD of this XML doc.
  virtual void SetStyleSheet(nglString type, nglString file); ///< Set the Style sheet of this xml doc.
  virtual nglString GetDTDName(); ///< Get the DTD Name of this XML doc.
  virtual nglString GetDTDFile(); ///< Get the DTD File name of this XML doc.
  virtual nglString GetStyleSheetType(); ///< Get the Style sheet type of this xml doc.
  virtual nglString GetStyleSheetFile(); ///< Get the Style sheet file name of this xml doc.
protected:
  virtual bool ParseXMLHeader(xmlLexer* pLexer, nglString& str);
private:
  nuiXMLNode* mpChild;
  nglString mDTDName;
  nglString mDTDFile;
  nglString mStyleSheetType;
  nglString mStyleSheetFile;

};

#endif // __nuiXML_h__
